CREATE OR REPLACE PROCEDURE WMS_SS_NOTIFY_OF_UPDATE IS
/******************************************************************************
   NAME:       WMS_SS_NOTIFY_OF_UPDATE
   PURPOSE:    

   REVISIONS:
   Ver        Date        Author           Description
   ---------  ----------  ---------------  ------------------------------------
   1.0        10/6/2005          1. Created this procedure.

   NOTES:

   Automatically available Auto Replace Keywords:
      Object Name:     WMS_SS_NOTIFY_OF_UPDATE
      Sysdate:         10/6/2005
      Date and Time:   10/6/2005, 9:37:27 AM, and 10/6/2005 9:37:27 AM
      Username:         (set in TOAD Options, Procedure Editor)
      Table Name:       (set in the "New PL/SQL Object" dialog)

******************************************************************************/
		email_to VARCHAR2(60);
		email_message VARCHAR(8096);
		crlf VARCHAR2(2) := chr(13)||chr(10);
		
		-- these are used in sending update notifications
		smtp_conn utl_smtp.connection;

		email_from VARCHAR2(60) := 'signup-alerts@school.edu';
		email_subject VARCHAR2(128) := 'Signup Sheet Tool Updated on ' || to_char(SYSDATE,'YYYY/MM/DD');
	
		mail_server VARCHAR2(60) := 'unauthenticated.amilserver.school.edu';
		mail_port NUMBER := 0; -- mail server port number here

		cursor getAffectedUsers
		is
			select 
			u.pk1, u.firstname, u.lastname, u.user_id, u.email, count(cm.pk1) as course_count
			from tmp_toc_buttons toc, course_main cm, course_users cu, users u
			where cm.pk1 = toc.crsmain_pk1
			and cu.crsmain_pk1 = cm.pk1
			and cu.row_status = 0
			and u.pk1 = cu.users_pk1
			and u.row_status = 0
			--and cm.course_name not in ('Mika''s Blackboard *Test*','Test Course CSW')
			group by u.pk1, u.firstname, u.lastname, u.user_id, u.email
			order by u.lastname;
			
		cursor getAffectedCoursesForUser(userPk NUMBER)
		is
			select 
			toc.crsmain_pk1, toc.button_label, cm.course_name
			from tmp_toc_buttons toc, course_main cm, course_users cu, users u
			where cm.pk1 = toc.crsmain_pk1
			and cu.crsmain_pk1 = cm.pk1
			and cu.row_status = 0
			and u.pk1 = cu.users_pk1
			and u.row_status = 0
			--and cm.course_name not in ('Mika''s Blackboard *Test*','Test Course CSW')
			and u.pk1 = userPk
			order by cm.course_name;
BEGIN

	 -- This code presumes that prior to the run the button info has been saved out to 
	 -- a temporary table called TMP_TOC_BUTTONS 
	 -- 
	 -- This code may be used to create and populate that table. Don't forget to drop it when done! 
	 -- create table tmp_toc_buttons as 
	 -- select crsmain_pk1, label as button_label 
	 -- from course_toc ctoc 
	 -- where ctoc.internal_handle = 'wms-wms_ss_stdnt-nav-1' 

	-- for each affected person 
	FOR user_info_rec IN getAffectedUsers 
	LOOP
		-- construct the message 
		email_to := user_info_rec.email;
		email_message := user_info_rec.firstname || ' ' || user_info_rec.lastname || ',' || crlf
		 	|| '    As a part of our planned release cycle we have updated the Signup Sheet '
			|| 'tool in Blackboard; this update fixes a minor bug and makes available a few '
			|| 'new features that have been requested. As a part of the update process the '
			|| 'course menu button that linked to the Signup Sheet tool had to be reset. From '
			|| 'your perspective this means that you may have to refresh the course menu ' 
			|| 'before that button will work again - in each affected course simply click '
			|| 'the Refresh link that''s near the bottom of the course menu. For you the '
			|| 'affected ';
		IF user_info_rec.course_count > 1 THEN
		   email_message := email_message || 'courses are:' || crlf;
		ELSE
		   email_message := email_message || 'course is:' || crlf;
		END IF;
		
		FOR courses_toc_rec IN getAffectedCoursesForUser(user_info_rec.pk1)
		LOOP
			email_message := email_message 
				|| '    ' || courses_toc_rec.course_name || ' (the ''' || courses_toc_rec.button_label || ''' button)' || crlf;
		END LOOP;
		email_message := email_message || crlf
			|| 'For future reference, the release cycle for Blackboard is: ' || crlf
			|| '    During fall reading period - patch release, Minor fixes and improvements, if necessary' || crlf
			|| '    At the end of Winter Study - patch release, Minor fixes and improvements, if necessary' || crlf
			|| '    During spring reading period - patch release, Minor fixes and improvements, if necessary' || crlf
			|| '    During the summer - major release, major changes / upgrades' || crlf
			|| crlf
			|| 'Thank you, and good luck with the rest of the semester!' || crlf
			|| crlf
			|| '    -Office of Information Technology';
			
		-- try to send it 
		 BEGIN -- try to email the alert, but don't cause a fuss if the email doesn't go through 
			 smtp_conn := utl_smtp.open_connection(mail_server,mail_port);
			 utl_smtp.helo(smtp_conn,mail_server);
			 utl_smtp.mail(smtp_conn,email_from);
			 utl_smtp.rcpt(smtp_conn,email_to); 
			 utl_smtp.open_data(smtp_conn);
			 utl_smtp.write_data(smtp_conn,'Subject: ' ||email_subject || crlf || crlf);
			 utl_smtp.write_data(smtp_conn,email_message);
			 utl_smtp.close_data(smtp_conn);
			 utl_smtp.quit(smtp_conn);
		   EXCEPTION
		     WHEN OTHERS THEN
			 	  NULL;
				  --RAISE;
		 END;
		
	END LOOP;	 
	 
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END WMS_SS_NOTIFY_OF_UPDATE;
/
