
CREATE TABLE BB_BB60.WMS_SS_GROUP
(
  ss_group_id NUMBER NOT NULL,
  date_created DATE DEFAULT SYSDATE NOT NULL,
  date_updated DATE DEFAULT SYSDATE NOT NULL,
  deleted VARCHAR2(1),
  owner_user_pk1 NUMBER NOT NULL,
  name VARCHAR2(64) NOT NULL,
  description VARCHAR2(255),
  max_total_slots_per_user NUMBER,
  max_pending_slots_per_user NUMBER,
  is_default VARCHAR2(1),
  CONSTRAINT PK_WMS_SS_GROUP PRIMARY KEY (ss_group_id)
)
TABLESPACE BB_BB60_DATA

CREATE TABLE BB_BB60.WMS_SS_FRAMEWORK
(
  ss_framework_id NUMBER NOT NULL,
  date_created DATE DEFAULT SYSDATE NOT NULL,
  date_updated DATE DEFAULT SYSDATE NOT NULL,
  deleted VARCHAR2(1),
  owner_user_pk1 NUMBER NOT NULL,
  ss_group_id NUMBER NOT NULL,
  name VARCHAR2(64) NOT NULL,
  description VARCHAR2(255),
  max_total_slots_per_user NUMBER,
  max_pending_slots_per_user NUMBER,
  change_alert_owner_flag VARCHAR2(1),
  imminent_alert_owner_flag VARCHAR2(1),
  signup_deadline DATE,
  ss_resource VARCHAR2(64),
  ss_location VARCHAR2(64),
  CONSTRAINT PK_WMS_SS_FRAMEWORK PRIMARY KEY (ss_framework_id)
)
TABLESPACE BB_BB60_DATA

CREATE TABLE BB_BB60.WMS_SS_ACCESS
(
  ss_access_id NUMBER NOT NULL,
  date_created DATE DEFAULT SYSDATE NOT NULL,
  date_updated DATE DEFAULT SYSDATE NOT NULL,
  deleted VARCHAR2(1),
  owner_user_pk1 NUMBER NOT NULL,
  ss_framework_id NUMBER NOT NULL,
  allowed_pk1 NUMBER NOT NULL,
  access_category VARCHAR2(8) NOT NULL,
  CONSTRAINT PK_WMS_SS_ACCESS PRIMARY KEY (ss_access_id)
)
TABLESPACE BB_BB60_DATA

CREATE TABLE BB_BB60.WMS_SS_BLOCK
(
  ss_block_id NUMBER NOT NULL,
  date_created DATE DEFAULT SYSDATE NOT NULL,
  date_updated DATE DEFAULT SYSDATE NOT NULL,
  deleted VARCHAR2(1),
  owner_user_pk1 NUMBER NOT NULL,
  ss_framework_id NUMBER NOT NULL,
  ss_block_set_id NUMBER NOT NULL,
  start_datetime DATE NOT NULL,
  end_datetime DATE NOT NULL,
  max_slots NUMBER NOT NULL,
  admin_comment VARCHAR2(512) DEFAULT ' ',
  CONSTRAINT PK_WMS_SS_BLOCK PRIMARY KEY (ss_block_id)
)
TABLESPACE BB_BB60_DATA

CREATE TABLE BB_BB60.WMS_SS_SLOT
(
  ss_slot_id NUMBER NOT NULL,
  date_created DATE DEFAULT SYSDATE NOT NULL,
  date_updated DATE DEFAULT SYSDATE NOT NULL,
  deleted VARCHAR2(1),
  updated_by_user_pk1 NUMBER NOT NULL,
  ss_block_id NUMBER NOT NULL,
  user_pk1 NUMBER NOT NULL,
  admin_comment VARCHAR2(512) DEFAULT ' ',
  CONSTRAINT PK_WMS_SS_SLOT PRIMARY KEY (ss_slot_id)
)
TABLESPACE BB_BB60_DATA

CREATE TABLE WMS_SS_ADMIN
(
  SS_ADMIN_ID         NUMBER                    NOT NULL,
  DATE_CREATED        DATE DEFAULT SYSDATE      NOT NULL,
  DATE_UPDATED        DATE DEFAULT SYSDATE      NOT NULL,
  DELETED             VARCHAR2(1),
  SS_FRAMEWORK_ID     NUMBER                    NOT NULL,
  ADMIN_USER_PK       NUMBER                    NOT NULL,
  GRANTED_BY_USER_PK  NUMBER                    NOT NULL,
  CONSTRAINT PK_WMS_SS_ADMIN PRIMARY KEY (ss_admin_id)
)
TABLESPACE BB_BB60_DATA



CREATE SEQUENCE wms_ss_ids
	START WITH 1
	INCREMENT BY 1
	NOMAXVALUE;


CREATE OR REPLACE VIEW BB_BB60.WMS_SS_VW_SLOT_ND
(ss_slot_id, 
date_created, 
date_updated, 
deleted, 
updated_by_user_pk1, 
ss_block_id, 
user_pk1,
admin_comment)
AS 
SELECT 
ss_slot_id, 
date_created, 
date_updated, 
deleted, 
updated_by_user_pk1, 
ss_block_id, 
user_pk1,
admin_comment
FROM BB_BB60.wms_ss_slot
WHERE deleted != 'Y';


CREATE OR REPLACE VIEW BB_BB60.WMS_SS_VW_BLOCK_ND
(ss_block_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
ss_framework_id, 
ss_block_set_id, 
start_datetime, 
end_datetime, 
max_slots,
admin_comment)
AS
SELECT 
ss_block_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
ss_framework_id, 
ss_block_set_id, 
start_datetime, 
end_datetime, 
max_slots,
admin_comment
FROM BB_BB60.wms_ss_block
WHERE deleted != 'Y';


CREATE OR REPLACE VIEW BB_BB60.WMS_SS_VW_BLOCK_USE
(ss_block_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
ss_framework_id, 
ss_block_set_id, 
start_datetime, 
end_datetime, 
max_slots,
admin_comment,
block_usage)
AS
SELECT 
ssb.ss_block_id, ssb.date_created, ssb.date_updated, 
ssb.deleted, ssb.owner_user_pk1, ssb.ss_framework_id, 
ssb.ss_block_set_id, ssb.start_datetime, ssb.end_datetime, 
ssb.max_slots, admin_comment, count(sss.ss_slot_id)
FROM WMS_SS_VW_BLOCK_ND ssb, WMS_SS_VW_SLOT_ND sss
WHERE ssb.ss_block_id = sss.ss_block_id(+)
GROUP BY
ssb.ss_block_id, ssb.date_created, ssb.date_updated, 
ssb.deleted, ssb.owner_user_pk1, ssb.ss_framework_id, 
ssb.ss_block_set_id, ssb.start_datetime, ssb.end_datetime, 
ssb.max_slots;

CREATE OR REPLACE VIEW BB_BB60.WMS_SS_VW_FRAMEWORK_ND
(ss_framework_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
ss_group_id, 
name, 
description, 
max_total_slots_per_user, 
max_pending_slots_per_user, 
change_alert_owner_flag,
imminent_alert_owner_flag,
signup_deadline,
ss_resource, 
ss_location)
AS
SELECT 
ss_framework_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
ss_group_id, 
name, 
description, 
max_total_slots_per_user, 
max_pending_slots_per_user, 
change_alert_owner_flag,
imminent_alert_owner_flag,
signup_deadline,
ss_resource, 
ss_location
FROM wms_ss_framework
WHERE deleted != 'Y';

CREATE OR REPLACE VIEW BB_BB60.WMS_SS_VW_GROUP_ND
(ss_group_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
name, 
description, 
max_total_slots_per_user, 
max_pending_slots_per_user,
is_default)
AS
SELECT 
ss_group_id, 
date_created, 
date_updated, 
deleted, 
owner_user_pk1, 
name, 
description, 
max_total_slots_per_user, 
max_pending_slots_per_user,
is_default
FROM wms_ss_group
WHERE deleted != 'Y';


CREATE OR REPLACE VIEW WMS_SS_VW_ADMIN_ND
(
SS_ADMIN_ID, 
DATE_CREATED, 
DATE_UPDATED, 
DELETED, 
SS_FRAMEWORK_ID, 
ADMIN_USER_PK, 
GRANTED_BY_USER_PK)
AS 
SELECT
  SS_ADMIN_ID,
  DATE_CREATED,
  DATE_UPDATED,
  DELETED,
  SS_FRAMEWORK_ID,
  ADMIN_USER_PK,
  GRANTED_BY_USER_PK
FROM
wms_ss_admin
WHERE
deleted != 'Y';



ALTER TABLE BB_BB60.WMS_SS_SLOT
  ADD ADMIN_COMMENT VARCHAR2(512) DEFAULT ' '

ALTER TABLE BB_BB60.WMS_SS_BLOCK
  ADD ADMIN_COMMENT VARCHAR2(512) DEFAULT ' '
  
