CREATE OR REPLACE PROCEDURE WMS_SS_DAILY_ALERTS IS
/******************************************************************************
   NAME:       WMS_SS_DAILY_ALERTS
   PURPOSE:    

   REVISIONS:
   Ver        Date        Author           Description
   ---------  ----------  ---------------  ------------------------------------
   1.0        9/12/2005   CSW       1. Created this procedure.
   2.0        9/12/2005   CSW       2. added admin comment support

   NOTES:

   Automatically available Auto Replace Keywords:
      Object Name:     WMS_SS_DAILY_ALERTS
      Sysdate:         9/12/2005
      Date and Time:   9/12/2005, 4:16:34 PM, and 9/12/2005 4:16:34 PM
      Username:         (set in TOAD Options, Procedure Editor)
      Table Name:       (set in the "New PL/SQL Object" dialog)

******************************************************************************/
		signee_user_id NUMBER;
		owner_user_id NUMBER;
		last_block_id NUMBER;
		last_framework_id NUMBER;
		
		-- this is how far in advance (in days) to check for upcoming signups
		signup_look_ahead NUMBER := 1.5;
		
		-- these are used in building the emails 
		signup_count NUMBER;
		block_count NUMBER;
		block_day VARCHAR(32);
		block_starttime VARCHAR(32);
		block_endtime VARCHAR(32);
		email_to VARCHAR2(60);
		email_message VARCHAR(8096);
		crlf VARCHAR2(2) := chr(13)||chr(10);
		
		-- and these in sending them
		smtp_conn utl_smtp.connection;

		email_from VARCHAR2(60) := 'signup-alerts@blackboard.williams.edu';
		--email_from VARCHAR2(60) := 'cwarren@williams.edu';
		email_subject VARCHAR2(128) := to_char(SYSDATE,'YYYY/MM/DD') || ' Upcoming BB signup-sheet appointments';
	
		mail_server VARCHAR2(60) := 'mailgate.williams.edu';
		mail_port NUMBER := 25;

                -- NOTE: table aliases refer to 'student', but that is just a
                -- legacy of earlier development. Anybody may be signed up,
                -- student or not. ALSO, owners re not necessarily instructors.

		cursor getSignees
		is
			select distinct 
				   ssslot.user_pk1 as signee_pk, 
				   studentu.user_id as signee_user_id, 
				   studentu.firstname as signee_firstname, 
				   studentu.lastname as signee_lastname, 
				   studentu.email as signee_email
			from
			wms_ss_vw_slot_nd ssslot, wms_ss_vw_block_nd ssblock, wms_ss_vw_framework_nd ssframework, users studentu
			where
			ssblock.ss_block_id = ssslot.ss_block_id
			and ssblock.start_datetime > SYSDATE
			and ssblock.start_datetime < SYSDATE+signup_look_ahead
			and ssframework.ss_framework_id = ssblock.ss_framework_id
			and studentu.pk1 = ssslot.user_pk1;
			
		cursor getOwners
		is
		  	select distinct 
				   ssblock.owner_user_pk1 as owner_pk, 
				   owneru.user_id as owner_user_id, 
				   owneru.firstname as owner_firstname, 
				   owneru.lastname as owner_lastname, 
				   owneru.email as owner_email
			from
			wms_ss_vw_slot_nd ssslot, wms_ss_vw_block_nd ssblock, wms_ss_vw_framework_nd ssframework, users owneru
			where
			ssblock.ss_block_id = ssslot.ss_block_id
			and ssblock.start_datetime > SYSDATE
			and ssblock.start_datetime < SYSDATE+signup_look_ahead
			and ssframework.ss_framework_id = ssblock.ss_framework_id
			and owneru.pk1 = ssblock.owner_user_pk1;
			
		cursor getSigneeAppointments (signee_pk NUMBER)
		is
			select
				ssframework.ss_framework_id as framework_id, 
				ssframework.name as framework_name, 
				ssframework.description as framework_description, 
				ssslot.ss_slot_id as slot_id, 
				studentu.firstname as signee_firstname, 
				studentu.lastname as signee_lastname, 
				studentu.user_id as signee_user_id, 
				ssslot.user_pk1 as signee_pk, 
				owneru.firstname as owner_firstname, 
				owneru.lastname as owner_lastname, 
				owneru.user_id as owner_user_id, 
				ssblock.owner_user_pk1 as owner_pk, 
				ssblock.start_datetime as block_start, 
				ssblock.end_datetime as block_end, 
				ssblock.admin_comment as block_comment, 
				trunc((ssblock.start_datetime - SYSDATE)*24,2) as hours_til
			from
			wms_ss_vw_slot_nd ssslot, wms_ss_vw_block_nd ssblock, wms_ss_vw_framework_nd ssframework, users studentu, users owneru 
			where
			ssblock.ss_block_id = ssslot.ss_block_id
			and ssblock.start_datetime > SYSDATE
			and ssblock.start_datetime < SYSDATE+signup_look_ahead
			and ssframework.ss_framework_id = ssblock.ss_framework_id
			and studentu.pk1 = ssslot.user_pk1
			and owneru.pk1 = ssblock.owner_user_pk1
			and ssslot.user_pk1=signee_pk
			order by ssblock.start_datetime, studentu.lastname, studentu.firstname;
			
		cursor getOwnerAppointments (owner_pk NUMBER)
		is
			select
				ssframework.ss_framework_id as framework_id, 
				ssframework.name as framework_name, 
				ssframework.description as framework_description, 
				ssslot.ss_slot_id as slot_id, 
				studentu.firstname as signee_firstname, 
				studentu.lastname as signee_lastname, 
				studentu.user_id as signee_user_id, 
				ssslot.user_pk1 as signee_pk, 
				owneru.firstname as owner_firstname, 
				owneru.lastname as owner_lastname, 
				owneru.user_id as owner_user_id, 
				ssblock.owner_user_pk1 as owner_pk, 
				ssblock.ss_block_id as block_id, 
				ssblock.start_datetime as block_start, 
				ssblock.end_datetime as block_end, 
				ssblock.admin_comment as block_comment, 
				trunc((ssblock.start_datetime - SYSDATE)*24,2) as hours_til
			from
			wms_ss_vw_slot_nd ssslot, wms_ss_vw_block_nd ssblock, wms_ss_vw_framework_nd ssframework, users studentu, users owneru 
			where
			ssblock.ss_block_id = ssslot.ss_block_id
			and ssblock.start_datetime > SYSDATE
			and ssblock.start_datetime < SYSDATE+signup_look_ahead
			and ssframework.ss_framework_id = ssblock.ss_framework_id
			and studentu.pk1 = ssslot.user_pk1
			and owneru.pk1 = ssblock.owner_user_pk1
			and ssblock.owner_user_pk1=owner_pk
			order by ssblock.start_datetime, studentu.lastname, studentu.firstname;
BEGIN

	 -- over all organization: first handle the signees/students, then handle the owners/instructors

	 -- SIGNEES/STUDENTS 
	 -- 1. get the list of user PKs (and names and emails) of people that have signed up for something in the next X hours/days 
	 FOR signee_rec IN getSignees LOOP
	 	 -- 2. for each user, get a list of their signups (blocks, ordered by start time) 
	 	 -- 3. compose and send an email to that person 
		 email_to := signee_rec.signee_email;
		 email_message := '';
		 signup_count := 0;
	 	 FOR signee_appointment_rec IN getSigneeAppointments(signee_rec.signee_pk) LOOP
	 		 signup_count := signup_count + 1;
		 	 -- what time (day, start, end) 
			 -- what framework (name, descr) 
			 block_day := trim(to_char(signee_appointment_rec.block_start,'DAY')) || ', ' || to_char(signee_appointment_rec.block_start,'YYYY/MM/DD'); 
			 block_starttime := ltrim(to_char(signee_appointment_rec.block_start,'HH:MI AM'),'0'); 
			 block_endtime := ltrim(to_char(signee_appointment_rec.block_end,'HH:MI AM'),'0'); 
			 email_message := email_message || '    * ' || block_day || ' from ' || block_starttime || ' to ' || block_endtime || ' for ' || signee_appointment_rec.framework_name || crlf;
 			 email_message := email_message || '        "' || signee_appointment_rec.framework_description || '"' || crlf; 
 			 email_message := email_message || '        "' || signee_appointment_rec.block_comment || '"' || crlf || crlf; 
		 END LOOP;

		 IF (signup_count > 1) THEN
		 	email_message := 'Your upcoming signups -' || crlf || crlf || email_message;
		 ELSE
		 	email_message := 'Your upcoming signup -' || crlf || crlf || email_message;
		 END IF;

		 --DBMS_OUTPUT.PUT_LINE('mail_server: ' || mail_server);
		 --DBMS_OUTPUT.PUT_LINE('mail_port: ' || mail_port);
		 --DBMS_OUTPUT.PUT_LINE('email_from: ' || email_from);
		 --DBMS_OUTPUT.PUT_LINE('email_to: ' || email_to);
		 --DBMS_OUTPUT.PUT_LINE('Subject: ' || 'Subject: ' ||email_subject || crlf || crlf);
		 --DBMS_OUTPUT.PUT_LINE('email_message: ' || email_message);
		 
		 BEGIN -- try to email the alert, but don't cause a fuss if the email doesn't go through 
			 smtp_conn := utl_smtp.open_connection(mail_server,mail_port);
			 utl_smtp.helo(smtp_conn,mail_server);
			 utl_smtp.mail(smtp_conn,email_from);
			 utl_smtp.rcpt(smtp_conn,email_to); 
			 utl_smtp.open_data(smtp_conn);
			 utl_smtp.write_data(smtp_conn,'Subject: ' ||email_subject || crlf || crlf);
			 utl_smtp.write_data(smtp_conn,email_message);
			 utl_smtp.close_data(smtp_conn);
			 utl_smtp.quit(smtp_conn);
		   EXCEPTION
		     WHEN OTHERS THEN
			 	  NULL;
				  --RAISE;
		 END;
	 	 
	 END LOOP;

	 -- OWNERS/INSTRUCTORS 
	 -- 4. get the list of user PKs (and names and emails) of people that own at least one block that starts in the next X days/hours and that has at least one signup
	 FOR owner_rec IN getOwners LOOP
		 -- 5. for each user, get their impending blocks (ordered by start time, with list of signees) 
		 -- 6. compose and send email to that person 
		 email_to := owner_rec.owner_email;
		 email_message := '';
		 signup_count := 0;
		 block_count := 0;
		 last_block_id := 0;
	 	 FOR owner_appointment_rec IN getOwnerAppointments(owner_rec.owner_pk) LOOP
	 		 signup_count := signup_count + 1;
			 IF (owner_appointment_rec.block_id != last_block_id) THEN
			 	 email_message := email_message || crlf;
				 block_day := trim(to_char(owner_appointment_rec.block_start,'DAY')) || ', ' || to_char(owner_appointment_rec.block_start,'YYYY/MM/DD'); 
				 block_starttime := ltrim(to_char(owner_appointment_rec.block_start,'HH:MI AM'),'0'); 
				 block_endtime := ltrim(to_char(owner_appointment_rec.block_end,'HH:MI AM'),'0'); 
				 --block_day := to_char(owner_appointment_rec.block_start,'DAY, YYYY/MM/DD'); 
				 --block_starttime := to_char(owner_appointment_rec.block_start,'HH:MI AM'); 
				 --block_endtime := to_char(owner_appointment_rec.block_end,'HH:MI AM'); 
				 email_message := email_message || '    * ' || block_day || ' from ' || block_starttime || ' to ' || block_endtime || ' for ' || owner_appointment_rec.framework_name || crlf;
	 			 email_message := email_message || '        "' || owner_appointment_rec.framework_description || '"' || crlf; 
 			 	 email_message := email_message || '        "' || owner_appointment_rec.block_comment || '"' || crlf; 
	 			 email_message := email_message || '        Signees:' || crlf; 
			 END IF;
			 email_message := email_message || '            ' || owner_appointment_rec.signee_firstname || ' ' || owner_appointment_rec.signee_lastname || crlf;
			 last_block_id := owner_appointment_rec.block_id;
		 END LOOP;

		 IF (signup_count > 1) THEN
		 	email_message := 'Your upcoming signups -' || crlf || email_message;
		 ELSE
		 	email_message := 'Your upcoming signup -' || crlf || email_message;
		 END IF;

		 --DBMS_OUTPUT.PUT_LINE('mail_server: ' || mail_server);
		 --DBMS_OUTPUT.PUT_LINE('mail_port: ' || mail_port);
		 --DBMS_OUTPUT.PUT_LINE('email_from: ' || email_from);
		 --DBMS_OUTPUT.PUT_LINE('email_to: ' || email_to);
		 --DBMS_OUTPUT.PUT_LINE('Subject: ' || 'Subject: ' ||email_subject || crlf || crlf);
		 --DBMS_OUTPUT.PUT_LINE('email_message: ' || email_message);
		 
		 BEGIN -- try to email the alert, but don't cause a fuss if the email doesn't go through 
			 smtp_conn := utl_smtp.open_connection(mail_server,mail_port);
			 utl_smtp.helo(smtp_conn,mail_server);
			 utl_smtp.mail(smtp_conn,email_from);
			 utl_smtp.rcpt(smtp_conn,email_to); 
			 utl_smtp.open_data(smtp_conn);
			 utl_smtp.write_data(smtp_conn,'Subject: ' ||email_subject || crlf || crlf);
			 utl_smtp.write_data(smtp_conn,email_message);
			 utl_smtp.close_data(smtp_conn);
			 utl_smtp.quit(smtp_conn);
		   EXCEPTION
		     WHEN OTHERS THEN
			 	  NULL;
				  --RAISE;
		 END;
	 	 
	 END LOOP;

	 
	 
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END WMS_SS_DAILY_ALERTS;
/
