/*
 * WmsEmailer.java
 *
 * Created on September 7, 2005, 4:26 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.email;

import java.util.*;
import java.io.*;
import javax.mail.*;
import javax.mail.internet.*;

/**
 * This is a simple email tool that uses javamail to send SMTP messages via a named mailhost. 
 * It does not do any error handling or validation or authentication - it's very bare bones.
 * For example:
 * <CODE>
 * WmsEmailer emailer = new WmsEmailer("mail.institution.edu");
 * String emailMessage = "This is the message body";
 * try {
 *    emailer.sendMessage("thesender@institution.edu",
 *            "therecipient@institution.edu",
 *            "A message subject",
 *            emailMessage);
 * } catch (SendFailedException e) {
 * }
 * </CODE>
 * @author cwarren
 */
public class WmsEmailer {

    private Properties mailServerProperties = new Properties();
    /**
     * The email addres for the From field for Signup Sheet alerts.
     * NOTE: these have been generalized - put your own info in here!
     */
    public static String alertsFromEmailAddress = "signup-alerts@school.edu";
    public static String generalFromEmailAddress = "blackboard-emails@school.edu";
    /* NOTE: this should be a mail server that does not require authentication */
    private static String mailServer = "mail.server";
    /**
     * Default constructor
     */
    public WmsEmailer()
    {
        mailServerProperties.put("mail.smtp.host", mailServer);
    }

    /**
     * Similar to the default constructor, but you may specify a mail server instead of using the default one.
     * @param mailServerName The name of the mail server that will be used to send the SMTP 
     * message. That server must not require authentication.
     */
    public WmsEmailer(String mailServerName)
    {
        mailServerProperties.put("mail.smtp.host", mailServerName);
    }

    /**
     * Methods
     * @return A Session object for the server associated with this emailer.
     */
    public Session getEmailSession()
    {
        Session s = Session.getInstance(mailServerProperties,null);
        return s;
    }
    
    /**
     * This is the main mthod of the emailer, the one that actually sends a message.
     * @param fromEmailAddr The email address to appear in the From field of the message; the address to which user replies are sent.
     * @param toEmailAddr The email address to appear in the To field of the message; the address to which the message will be sent.
     * @param mailSubject The message subject
     * @param mailText The body of the message
     * @throws javax.mail.internet.AddressException 
     * @throws javax.mail.MessagingException 
     */
    public void sendMessage(String fromEmailAddr,
                            String toEmailAddr,
                            String mailSubject,
                            String mailText)
        throws AddressException, MessagingException
    {
        Session s = this.getEmailSession();
        InternetAddress from = new InternetAddress(fromEmailAddr);
        InternetAddress to = new InternetAddress(toEmailAddr);

        MimeMessage message = new MimeMessage(s);
        message.setFrom(from);
        message.addRecipient(Message.RecipientType.TO, to);

        message.setSubject(mailSubject);
        message.setText(mailText);
        Transport.send(message);
    }  
} 

