/*
 * Admin.java
 *
 * Created on January 4, 2006, 2:53 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.util.Calendar;
import java.util.Date;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;

/**
 * This represents an admin record, which associates a signup sheet framework 
 * with a bb user (by the bb PK1). Admin determines who can modify the sheet and
 * see the details (e.g. look at the blocks, sign people up, etc.). The sheet 
 * owner always gets Admin, and anyone with Admin can give such priveleges to 
 * others. 
 * @author cwarren
 */
public class Admin {
    
    private int adminId;
    private java.util.Date dateCreated, dateUpdated;
    private int frameworkId, adminUserPk, grantedByUserPk;
    private String userId;
    
    /** Creates a new instance of Admin */
    public Admin() {
    }

    /**
     * Creates a new instance of Admin
     * @param frameworkId The ID of the framework to which administrative privileges are granted
     * @param adminUserPk The BB user PK of the person who is getting the admin privileges (i.e. the 
     * grantee)
     * @param grantedByUserPk The BB user PK of the person who is granting the privileges (i.e. the grantor)
     */
    public Admin(int frameworkId, int adminUserPk, int grantedByUserPk) {
        this.setFrameworkId(frameworkId);
        this.setAdminUserPk(adminUserPk);
        this.setGrantedByUserPk(grantedByUserPk);
    }

    /**
     * Creates a new instance of Admin from a ResultSet that contains a singlerow and
     * all fields from the WMS_SS_ADMIN table
     * @param res a ResultSet that contains a singlerow and
     * all fields from the WMS_SS_ADMIN table
     * @throws java.sql.SQLException 
     */
    public Admin(ResultSet res) 
        throws SQLException
    {
        this.adminId = res.getInt("SS_ADMIN_ID");
        this.dateCreated = res.getTimestamp("DATE_CREATED");
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setFrameworkId(res.getInt("SS_FRAMEWORK_ID"));
        this.setAdminUserPk(res.getInt("ADMIN_USER_PK"));
        this.setGrantedByUserPk(res.getInt("GRANTED_BY_USER_PK"));
        try
        {
            res.findColumn("USER_ID");
            this.setUserId(res.getString("USER_ID"));
        }
        catch (SQLException e)
        {
            this.setUserId("");
        }
        
    }

    //-------------------------------------------------------------------------

    /**
     * Removes an admin with the given ID from the DB (marks if for deletion really, but 
     * undeleting has to happen at the DB level - when this is called the Block is effectively 
     * gone).
     * @param adminId the ID of an Admin record
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteByAdminId (int adminId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        SqlUtil.runSql("UPDATE wms_ss_admin SET deleted = 'Y' WHERE ss_admin_id = "+Integer.toString(adminId));
    }

    //-------------------------------------------------------------------------
    
    /**
     * Save the current object to the DB as a new admin record
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
        this.adminId = newId;
        Calendar cal = Calendar.getInstance();
        this.dateCreated = cal.getTime();
        this.dateUpdated = cal.getTime();
        String sqlText = 
                "INSERT INTO wms_ss_admin ("+
                    " ss_admin_id, date_created, date_updated, " +
                    " deleted, " +
                    "ss_framework_id, admin_user_pk, granted_by_user_pk) " +
               " VALUES ("
                    +Integer.toString(this.getAdminId())+"," // ss_admin_id
                    +"SYSDATE," // date_created
                    +"SYSDATE," // date_updated
                    +"'N'," // deleted
                    +Integer.toString(this.getFrameworkId())+"," //ss_framework_id
                    +Integer.toString(this.getAdminUserPk())+"," // allowed_pk1
                    +Integer.toString(this.getGrantedByUserPk()) // access_category
                +")";
        SqlUtil.runSql(sqlText);
    }    
    
    //-------------------------------------------------------------------------

    public int getAdminId() {
        return adminId;
    }

    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    public int getFrameworkId() {
        return frameworkId;
    }

    public void setFrameworkId(int frameworkId) {
        this.frameworkId = frameworkId;
    }

    public int getAdminUserPk() {
        return adminUserPk;
    }

    public void setAdminUserPk(int adminUserPk) {
        this.adminUserPk = adminUserPk;
    }

    public int getGrantedByUserPk() {
        return grantedByUserPk;
    }

    public void setGrantedByUserPk(int grantedByUserPk) {
        this.grantedByUserPk = grantedByUserPk;
    }

    public String getUserId() {
        if (userId.length() < 1)
        {
            String sqlText = "SELECT user_id FROM users WHERE pk1 = "+Integer.toString(this.getAdminUserPk());
            try
            {
                this.setUserId(SqlUtil.runSqlGetStringField(sqlText, "user_id"));
            }
            catch (Exception exc)
            {
            }
        }
        return userId;
    }
    
    private void setUserId(String userId) {
        this.userId = userId;
    }
    
}
