/*
 * Block.java
 *
 * Created on August 18, 2005, 1:01 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.util.Calendar;
import java.util.Date;
import wms.ss.data.SlotSignup;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import wms.ss.display.HtmlCalendarTool;
import wms.email.WmsEmailer;
import blackboard.db.*;

/**
 * This object represents a block of time for which users may sign up. It is associated 
 * with a Framework, and in turn has SlotSignups associated with it.
 * @author cwarren
 */
public class Block {
    
    private int blockId;
    private java.util.Date dateCreated, dateUpdated;
    private int ownerUserPk1, frameworkId, blockSetId, maxSlots, usedSlots;
    private java.util.Date startDatetime, endDatetime;
    private String adminComment;
    
    /**
     * This one of the various utility methods that retrieves a Block from the database, based 
     * on some unique identifier (the ID of a SlotSignup in this case).
     * @param slotSignupId The ID of a SlotSingup object
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The Block with which the indicated SlotSignup is associated.
     */
    public static Block loadBySlotSignupId (int slotSignupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return createFromSql("SELECT ssblock.* " +
                "FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot " +
                "WHERE ssblock.ss_block_id = ssslot.ss_block_id " +
                "AND ssslot.ss_slot_id = "+Integer.toString(slotSignupId));
    }

    /**
     * This one of the various utility methods that retrieves a Block from the database, based 
     * on some unique identifier (the ID of a Block in this case).
     * @param blockId The ID of the Block to get
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The Block with specified ID
     */
    public static Block loadByBlockId (int blockId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return createFromSql("SELECT * FROM wms_ss_vw_block_nd WHERE ss_block_id = "+Integer.toString(blockId));
    }

    /**
     * Removes a block with the given ID from the DB (marks if for deletion really, but 
     * undeleting has to happen at the DB level - when this is called the Block is effectively 
     * gone). All SlotSignups associated with this Block are also deleted, and for each one an 
     * email is sent to the user that signed up indicating that their signup was deleted.
     * @param blockId The ID of the Block to delete.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteByBlockId (int blockId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        deleteByBlockId (blockId, "the time block was deleted");
    }
    
    /**
     * Removes a block with the given ID from the DB (marks if for deletion really, but 
     * undeleting has to happen at the DB level - when this is called the Block is effectively 
     * gone). All SlotSignups associated with this Block are also deleted, and for each one an 
     * email with a custom message/reason is sent to the user that signed up indicating that 
     * their signup was deleted.
     * @param blockId The ID of the Block to delete
     * @param removalReason The reason (as a short phrase) for the deletion.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteByBlockId (int blockId, String removalReason)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT * FROM wms_ss_vw_slot_nd WHERE ss_block_id="+Integer.toString(blockId);
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        while(res.next())
        {
            SlotSignup ss = new SlotSignup(res);
            if ((removalReason != null) && (removalReason.length() > 0))
            {
                ss.emailRemoveAlertToUser(removalReason);
            }
            else
            {
                ss.emailRemoveAlertToUser();
            }
            SlotSignup.deleteBySlotSignupId(ss.getSlotSignupId());
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        SqlUtil.runSql("UPDATE wms_ss_block SET deleted = 'Y' WHERE ss_block_id = "+Integer.toString(blockId));
    }

    /** private static methods */
    private static Block createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        
        Block b = new Block();
        if (res.next())
        {
            b = new Block(res);
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return b;
    }    
    
    /**
     * Creates an empty Block object, not associated with anything in the DB.
     */
    public Block() {
    }

    /**
     * Creates a fleshed out block from a java.sql.ResultSet. The block is associated with a 
     * DB record.
     * @param res The java.sql.ResultSet that contains the query results necessary to create the block.
     * @throws java.sql.SQLException 
     */
    public Block(ResultSet res) 
        throws SQLException
    {
        this.blockId = res.getInt("SS_BLOCK_ID");
        this.dateCreated = res.getTimestamp("DATE_CREATED");
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setOwnerUserPk1(res.getInt("OWNER_USER_PK1")); 
        this.setFrameworkId(res.getInt("SS_FRAMEWORK_ID"));
        this.setBlockSetId(res.getInt("SS_BLOCK_SET_ID")); 
        this.setStartDatetime(res.getTimestamp("START_DATETIME"));
        this.setEndDatetime(res.getTimestamp("END_DATETIME"));
        this.setMaxSlots(res.getInt("MAX_SLOTS"));
        this.setAdminComment(res.getString("ADMIN_COMMENT"));
        try
        {
            res.findColumn("BLOCK_USAGE");
            this.setUsedSlots(res.getInt("BLOCK_USAGE"));
        }
        catch (SQLException e)
        {
            this.setUsedSlots(-1);
        }
    }

    /**
     * Updates the DB to make it match the current info in this Block object. Sets the 
     * date_updated field as appropriate.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void save()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "UPDATE wms_ss_block "+
            " SET "+
            " date_updated               = SYSDATE, "+
            " start_datetime              = to_date('"+SqlUtil.dateFormatter.format(this.getStartDatetime())+"','"+SqlUtil.dateFormatStringForDB+"'), "+
            " end_datetime                = to_date('"+SqlUtil.dateFormatter.format(this.getEndDatetime())+"','"+SqlUtil.dateFormatStringForDB+"'), "+
            " max_slots                   = "+Integer.toString(this.getMaxSlots())+","+
            " admin_comment               = '"+this.getAdminComment().replaceAll("'", "''")+"'"+
            " WHERE  ss_block_id = "+Integer.toString(this.getBlockId());
        
        SqlUtil.runSql(sqlText);
    }

    /**
     * Creates a new DB record based on this Block object - sets the ID, date_created, and 
     * date_updated fields as appropriate. Does not create the block if there's a time 
     * conflict on this sheet.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        if (! this.hasTimeConflictInThisFramework())
        {
            int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
            this.blockId = newId;
            Calendar cal = Calendar.getInstance();
            this.dateCreated = cal.getTime();
            this.dateUpdated = cal.getTime();
            if (this.getAdminComment() == null)
            {
                this.setAdminComment(" ");
            }
            String sqlText = "INSERT INTO wms_ss_block ("+
                    " ss_block_id, date_created, date_updated, " +
                    " deleted, owner_user_pk1, ss_framework_id, " +
                    " ss_block_set_id, start_datetime, end_datetime, " +
                    " max_slots, admin_comment) " +
                   " VALUES ("
                        +Integer.toString(this.getBlockId())+"," // ss_block_id
                        +"SYSDATE," // date_created
                        +"SYSDATE," // date_updated
                        +"'N'," // deleted
                        +Integer.toString(this.getOwnerUserPk1())+"," // owner_user_pk1
                        +Integer.toString(this.getFrameworkId())+"," //ss_framework_id
                        +Integer.toString(this.getBlockSetId())+"," // ss_block_set_id
                        +"to_date('"+SqlUtil.dateFormatter.format(this.getStartDatetime())+"','"+SqlUtil.dateFormatStringForDB+"')," // start_datetime
                        +"to_date('"+SqlUtil.dateFormatter.format(this.getEndDatetime())+"','"+SqlUtil.dateFormatStringForDB+"')," // end_datetime
                        +Integer.toString(this.getMaxSlots())+"," // max_slots
                        +"'"+this.getAdminComment().replaceAll("'", "''")+"'" // max_slots
                    +")";
            SqlUtil.runSql(sqlText);
        }
    }
    
    /**
     * Checks to see if this block has a time conflict on this sheet.
     * @return true if at least one conflict exists, false otherwise.
     */
    public boolean hasTimeConflictInThisFramework()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String formattedStartDatetime = "to_date('"+SqlUtil.checkConflictDateFormatter.format(this.getStartDatetime())+"','"+SqlUtil.dateFormatStringForDBConflictChecking+"')";
        String formattedEndDatetime = "to_date('"+SqlUtil.checkConflictDateFormatter.format(this.getEndDatetime())+"','"+SqlUtil.dateFormatStringForDBConflictChecking+"')";

        String sqlText = "SELECT count(*) AS conflict_count " +
                "FROM wms_ss_vw_block_nd " +
                "WHERE ss_framework_id="+Integer.toString(this.getFrameworkId())+" AND "+
                "(   (trunc(start_datetime,'MI') <= "+formattedStartDatetime+" AND trunc(end_datetime,'MI') >= "+formattedEndDatetime+") " +
                " OR (trunc(start_datetime,'MI') >= "+formattedStartDatetime+" AND trunc(start_datetime,'MI') < "+formattedEndDatetime+") " +
                " OR (trunc(end_datetime,'MI') > "+formattedStartDatetime+" AND trunc(end_datetime,'MI') <= "+formattedEndDatetime+") " +
                ")";
        
        //if (1==1) { throw new RuntimeException(sqlText); } // serious debugging hack
        
        int numConflicts = SqlUtil.runSqlGetIntField(sqlText, "conflict_count");
        
        return (numConflicts!=0);
    }
    
    
    /**
     * Gets the ID of the Block
     * @return The ID of this Block
     */

    public int getBlockId() {
        return blockId;
    }

    /**
     * Gets the date-time this Block was first put in the DB.
     * @return The date-time this Block was first put in the DB.
     */
    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Gets the date-time this Block's DB record was last changed.
     * @return The date-time this Block's DB record was last changed.
     */
    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Set the date-time this Block's DB record was last changed.
     * @param dateUpdated The date-time this Block's DB record was last changed.
     */
    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    /**
     * Get the BB pk of the user the owns this.
     * @return The BB pk of the user that owns this.
     */
    public int getOwnerUserPk1() {
        return ownerUserPk1;
    }

    /**
     * Set the BB pk of the user that owns this
     * @param ownerUserPk1 The BB pk of the user that is the owner of this
     */
    public void setOwnerUserPk1(int ownerUserPk1) {
        this.ownerUserPk1 = ownerUserPk1;
    }

    /**
     * Get the ID of Framework for this Block
     * @return The ID of the Framework with which this block is associated
     */
    public int getFrameworkId() {
        return frameworkId;
    }

    /**
     * Set the framework for this block
     * @param frameworkId The ID of the Framework of which this block is a part
     */
    public void setFrameworkId(int frameworkId) {
        this.frameworkId = frameworkId;
    }

    /**
     * Get the set ID of this block. The blocks with a given Set ID were all created at the 
     * same time. The functionality this was initially designed to support was never 
     * implemented, so the set ID is not actually used for anything.
     * @return The ID of the set of which this Block is a part
     * @deprecated The functionality this was initially designed to support was never 
     * implemented, so the set ID is not actually used for anything.
     */
    public int getBlockSetId() {
        return blockSetId;
    }

    /**
     * Set the set ID for this block.
     * @param blockSetId The ID of the set of which this block is (will be) a part
     * @deprecated The functionality this was initially designed to support was never 
     * implemented, so the set ID is not actually used for anything.
     */
    public void setBlockSetId(int blockSetId) {
        this.blockSetId = blockSetId;
    }

    /**
     * Get the maximum number of distinct users that are allowed to sign up for this block.
     * @return The maximum number of distinct users that are allowed to sign up for this block.
     */
    public int getMaxSlots() {
        return maxSlots;
    }

    /**
     * Set t he maximum number of distinct users that are allowed to sign up for this block.
     * @param maxSlots The maximum number of distinct users that are allowed to sign up for this block.
     */
    public void setMaxSlots(int maxSlots) {
        this.maxSlots = maxSlots;
    }

    /**
     * This returns the number of people that have signed up for this block. Call 
     * computeBlockUsage before this to ensure accurate info.
     * @return The number of users signed up for this Block
     */
    public int getUsedSlots() {
        return usedSlots;
    }

    /**
     * Set the number of users signed up for this Block. This does not affect the DB record, only 
     * the in-memory object - UsedSlots is effectively read-only wrt DB data.
     * @param usedSlots The number of users signed up for this Block.
     */
    public void setUsedSlots(int usedSlots) {
        this.usedSlots = usedSlots;
    }
    /**
     * Gets the date and time the block starts
     * @return the date and time the block starts
     */
    public java.util.Date getStartDatetime() {
        return startDatetime;
    }

    /**
     * Set the date and time the block starts
     * @param startDatetime The date and time the block starts
     */
    public void setStartDatetime(java.util.Date startDatetime) {
        this.startDatetime = startDatetime;
    }

    // sets the year, month, and day portion of the startDatetime, leaving the time alone
    /**
     * This sets only the date (year, month, day) part of the start date time for this block.
     * @param startDate The date the block starts
     */
    public void setStartDateOnly(java.util.Date startDate) {
        Calendar newcal = Calendar.getInstance();
        newcal.setTime(startDate);
        Calendar cal = Calendar.getInstance();
        cal.setTime(this.startDatetime);
        cal.set(Calendar.YEAR,newcal.get(Calendar.YEAR));
        cal.set(Calendar.MONTH,newcal.get(Calendar.MONTH));
        cal.set(Calendar.DAY_OF_MONTH,newcal.get(Calendar.DAY_OF_MONTH));
        this.startDatetime = cal.getTime();
    }
    // as above, but passing in vals directly
    /**
     * This sets only the date (year, month, day) part of the start date time for this block
     * @param year The year the block starts
     * @param month the month the block starts
     * @param dayOfMonth the day of month the block starts
     */
    public void setStartDateOnly(int year, int month, int dayOfMonth) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(this.startDatetime);
        cal.set(Calendar.YEAR,year);
        cal.set(Calendar.MONTH,month);
        cal.set(Calendar.DAY_OF_MONTH,dayOfMonth);
        this.startDatetime = cal.getTime();
    }

    /**
     * Get the date and time the block ends
     * @return The date and time the block ends
     */
    public java.util.Date getEndDatetime() {
        return endDatetime;
    }

    /**
     * Set the date and time the block ends
     * @param endDatetime The date and time the block ends
     */
    public void setEndDatetime(java.util.Date endDatetime) {
        this.endDatetime = endDatetime;
    }

    // sets the year, month, and day portion of the endDatetime, leaving the time alone
    /**
     * This sets only the date (year, month, day) part of the end date time for this block
     * @param endDate The date the block ends
     */
    public void setEndDateOnly(java.util.Date endDate) {
        Calendar newcal = Calendar.getInstance();
        newcal.setTime(endDate);
        Calendar cal = Calendar.getInstance();
        cal.setTime(this.endDatetime);
        cal.set(Calendar.YEAR,newcal.get(Calendar.YEAR));
        cal.set(Calendar.MONTH,newcal.get(Calendar.MONTH));
        cal.set(Calendar.DAY_OF_MONTH,newcal.get(Calendar.DAY_OF_MONTH));
        this.endDatetime = cal.getTime();
    }
    // as above, but passing in vals directly
    /**
     * This sets only the date (year, month, day) part of the end date time for this block
     * @param year The year the block ends
     * @param month The month the block ends
     * @param dayOfMonth The day of the month the block ends
     */
    public void setEndDateOnly(int year, int month, int dayOfMonth) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(this.endDatetime);
        cal.set(Calendar.YEAR,year);
        cal.set(Calendar.MONTH,month);
        cal.set(Calendar.DAY_OF_MONTH,dayOfMonth);
        this.endDatetime = cal.getTime();
    }
    
    // does a DB query to figure out the current usage and sets the usage field accordingly
    /**
     * This makes the block do the appropriate DB checks to see to total number of users 
     * signed up for it - the UsedSlots attribute is set as appropriate.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void computeBlockUsage() 
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT ssblock.ss_block_id, count(ssslot.ss_slot_id) AS block_usage" +
                " FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot" +
                " WHERE ssblock.ss_block_id = " + Integer.toString(this.getBlockId()) +
                " AND ssslot.ss_block_id = ssblock.ss_block_id" +
                " GROUP BY ssblock.ss_block_id";                
/*        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        if (res.next())
        {
            this.setUsedSlots(res.getInt("block_usage"));
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
 */
        this.setUsedSlots(SqlUtil.runSqlGetIntField(sqlText, "block_usage"));
    }
    
    /**
     * Send an email to with the given subject and body to every person that signed up for this 
     * block. Email appears to come from WmsEmailer.alertsFromEmailAddress.
     * @param subject The subject line of the email to be sent
     * @param message The body of the email sent.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @see WmsEmailer.alertsFromEmailAddress
     */
    public void sendEmailToSignees(String subject, String message)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        SlotSignupList ssl = SlotSignupList.loadByBlockId(this.getBlockId());
        for (int i = 0; i < ssl.size(); i++)
        {
            SlotSignup ss = (SlotSignup)(ssl.get(i));
            ss.emailMessageFromBlockOwnerToUser(subject, message);
        }
    }

    /**
     * Send an email to with the given subject and body to the owner of this 
     * block. Email appears to come from WmsEmailer.alertsFromEmailAddress.
     * @param subject The subject line of the email to be sent
     * @param message The body of the email sent.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @see WmsEmailer.alertsFromEmailAddress
     */
    public void sendEmailToOwner(String subject, String message)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
            WmsEmailer emailer = new WmsEmailer();
            String sqlText = "SELECT email FROM users WHERE pk1="+Integer.toString(this.getOwnerUserPk1());
            String ownerEmail = SqlUtil.runSqlGetStringField(sqlText, "EMAIL");
            try
            {
                emailer.sendMessage(WmsEmailer.generalFromEmailAddress,
                                    ownerEmail,
                                    subject,
                                    message);
            }
            catch (Exception e)
            {
            }
    }    
    
    /**
     * Get the date (year, month, day) of the block as a string suitable for display
     * @return The date on which this block starts, in YYYY/MM/DD format.
     */
    public String getBlockDateStr()
    {
        return HtmlCalendarTool.downToDayDateFormatter.format(this.getStartDatetime());
    }
    
    /**
     * Get the timing (start time - end time) of this block as a string suitable for display.
     * @return The timing for this block in HH:MI AM - HH:MI AM format.
     */
    public String getBlockTimingStr()
    {
        return HtmlCalendarTool.hourMinAmPmFormatter.format(this.getStartDatetime())+"-"+HtmlCalendarTool.hourMinAmPmFormatter.format(this.getEndDatetime());
    }

    public String getAdminComment() {
        return adminComment;
    }

    public void setAdminComment(String adminComment) {
        this.adminComment = adminComment;
    }
}
    