/*
 * BlockList.java
 *
 * Created on August 19, 2005, 9:31 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;
import java.sql.*;

/**
 * This object is used to manage a list of Block objects. Generally is it not instantiated 
 * directly but instead is created through one of the many loadByX static methods, each of 
 * which returns a BlockList object. Access to the individual Blocks in the BlockList is 
 * done through the various ArrayList methods.
 * @author cwarren
 */
public class BlockList extends java.util.ArrayList {
    
    private static String dataSourceTable = "wms_ss_vw_block_nd";

    /**
     * Gets a list of all the Blocks for which a user has signed up.
     * UNIMPLEMENTED!
     * @param userPk1 A BB user pk
     * @return a BlockList
     */
    public static BlockList loadByUserId (int userPk1)
    {
       return loadByUserId (userPk1, null, null);
    }

    /**
     * Gets a list of all the Blocks for which a user has signed up, which start and end 
     * within a given time range.
     * UNIMPLEMENTED!
     * @param userPk1 A BB user pk
     * @param startDate A datetime
     * @param endDate A date time
     * @return a BlockList
     */
    public static BlockList loadByUserId (int userPk1, Date startDate, Date endDate)
    {
        BlockList bl = new BlockList();

        return bl;
    }

    /**
     * Gets a list of all the Blocks owned by a given user.
     * UNIMPLEMENTED!
     * @param userPk1 A BB user pk
     * @return a BlockList
     */
    public static BlockList loadByOwnerUserId (int userPk1)
    {
        return loadByOwnerUserId (userPk1, null, null);
    }

    /**
     * Gets a list of all the Blocks that start and end within a specified time range and which
     * are owned by a given user.
     * UNIMPLEMENTED!
     * @param userPk1 A BB user pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @return A BlockList
     */
    public static BlockList loadByOwnerUserId (int userPk1, Date startDate, Date endDate)
    {
        BlockList bl = new BlockList();

        return bl;
    }

    /**
     * Gets a list of all the Blocks in a Framework in the SheetGroup with the given ID.
     * UNIMPLEMENTED!
     * @param groupId The ID of a SheetGroup
     * @return A BlockList
     * @see SheetGroup
     */
    public static BlockList loadByGroupId (int groupId)
    {
        return loadByGroupId (groupId, null, null);
    }

   /**
    * Gets a list of all the Blocks in a Framework in the SheetGroup with the given ID, and
    * the Blocks are starting and ending within a specified time range
    * UNIMPLEMENTED!
    * @param groupId The ID of a group of Blocks
    * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
    * there's no early limit.
    * @param endDate A datetime - the latest time possible of a Block in this list. If null then
    * there's no late limit.
    * @return A BlockList
    */
   public static BlockList loadByGroupId (int groupId, Date startDate, Date endDate)
    {
        BlockList bl = new BlockList();

        return bl;
    }

    /**
     * Gets a lit of all Blocks in a given Framework.
     * @param frameworkId The ID of a Framework
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A BlockList
     */
    public static BlockList loadByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadByFrameworkId (frameworkId, null, null);
    }

    /**
     * Gets a lit of all Blocks in a given Framework that start and end within a specified
     * datetime range.
     * @param frameworkId A Framework ID
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A BlockList
     */
    public static BlockList loadByFrameworkId (int frameworkId, Date startDate, Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "select ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment, count(ssslot.ss_slot_id) as BLOCK_USAGE "+
            "FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot "+
            "WHERE  ssblock.ss_framework_id = "+Integer.toString(frameworkId) + " " +
            " AND ssblock.ss_block_id = ssslot.ss_block_id(+) ";
        sqlText = SqlUtil.appendDateRangeCheck(sqlText, "ssblock.start_datetime", "ssblock.end_datetime", startDate, endDate);
        sqlText = sqlText + " GROUP BY ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment";
        sqlText = sqlText + " ORDER BY ssblock.start_datetime";

        return BlockList.createFromSql(sqlText);
    }

    // a list of all blocks in the given framework that the user can access - i.e. blocks in which they're currently signed up and blocks that have space
    /**
     * Get all Blocks in a given Framework for which a given User has signed up.
     * @param userPk1 A BB user pk
     * @param frameworkId A Framework ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A BlockList
     */
    public static BlockList loadByUserIdFrameworkId (int userPk1, int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "SELECT ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment, count(ssslot.ss_slot_id) as BLOCK_USAGE "+
            "FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot  "+
            "WHERE  ssblock.ss_framework_id = "+Integer.toString(frameworkId)+" "+
            " AND ssblock.ss_block_id = ssslot.ss_block_id(+) "+
            " AND NOT EXISTS (select ss_slot_id from wms_ss_vw_slot_nd sss2 where sss2.ss_block_id = ssblock.ss_block_id and sss2.user_pk1="+Integer.toString(userPk1)+") "+
            "GROUP BY ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment "+
            //"HAVING ((count(ssslot.ss_slot_id) < ssblock.max_slots) OR (ssblock.max_slots=-1)) "+
            "union "+
            "SELECT ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment, -1 as BLOCK_USAGE "+
            "FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot  "+
            "WHERE  ssblock.ss_framework_id = "+Integer.toString(frameworkId)+" "+
            " AND ssblock.ss_block_id = ssslot.ss_block_id "+
            " AND ssslot.user_pk1 = "+Integer.toString(userPk1)+" "+
            " ORDER BY start_datetime";

        return BlockList.createFromSql(sqlText);
    }

    /**
     * Load all Blocks in any Framework associated with the given Course
     * UNIMPLEMENTED
     * @param courseId A BB course pk
     * @return A BlockList
     */
    public static BlockList loadByCourseId (int courseId)
    {
        return loadByCourseId (courseId, null, null);
    }

    /**
     * Load all Blocks in any Framework associated with the given Course and which start and
     * end within a specified time range.
     * UNIMPLEMENTED
     * @param courseId A BB course pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @return A BlockList
     */
    public static BlockList loadByCourseId (int courseId, Date startDate, Date endDate)
    {
        BlockList bl = new BlockList();

        return bl;
    }

    /**
     * Get a list of all Blocks with the given Block group ID; all blocks created in the same 
     * action are in the same group and are given a unique ID.
     * UNIMPLEMENTED
     * @param blockGroupId The ID of a group of Blocks
     * @return A BlockList
     */
    public static BlockList loadByBlockGroupId (int blockGroupId)
    {
        return loadByBlockGroupId(blockGroupId,null,null);
    }

    /**
     * Get a list of all Blocks with the given Block group ID and which start and end within a
     * specified time range; all blocks created in the same action are in the same group and 
     * are given a unique ID.
     * UNIMPLEMENTED
     * @param blockGroupId The ID of a group of Blocks
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @return A BlockList
     */
    public static BlockList loadByBlockGroupId (int blockGroupId, Date startDate, Date endDate)
    {
        BlockList bl = new BlockList();

        return bl;
    }

    
    /**
     * Returns a list of all blocks which are owned by the specified user and which 
     * have at least one signup.
     * @param userPk1 A BB user pk - the pk of the owner
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A BlockList; all blocks which are owned by the specified user and which 
     * have at least one signup.
     */
    public static BlockList loadUsedByOwnerUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByOwnerUserId (userPk1, null, null);
    }

    /**
     * Returns a list of all blocks which are owned by the specified user and which 
     * have at least one signup, and which start and end within the given datetime 
     * range.
     * @param userPk1 A BB user pk - the pk of the owner
     * @param startDate A datetime - the earliest starting time of a Block in the returned list. If null
     * then there's no early limit.
     * @param endDate A datetime - the latest ending time of a Block in the returned list. If null
     * then there's no early limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of all blocks which are owned by the specified user and which 
     * have at least one signup, and which start and end within the given datetime 
     * range.
     */
    public static BlockList loadUsedByOwnerUserId (int userPk1, Date startDate, Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByUserIdOwnerIdAndCourseId (0, userPk1, 0, startDate, endDate);
    }

    /**
     * Gets a list of all Blocks for which the given user has signed up.
     * @param userPk1 A BB user pk
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of all Blocks for which the given user has signed up.
     */
    public static BlockList loadUsedByUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByUserId (userPk1, null, null);
    }

    /**
     * Gets a list of all Blocks for which the given user has signed up and which start 
     * no earlier than the specified start datetime and end no later than the specified 
     * end datetime.
     * @param userPk1 A BB user pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of all Blocks for which the given user has signed up and which start 
     * no earlier than the specified start datetime and end no later than the specified 
     * end datetime.
     */
    public static BlockList loadUsedByUserId (int userPk1, Date startDate, Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByUserIdOwnerIdAndCourseId (userPk1, 0, 0, startDate, endDate);
}

    /**
     * Gets a list of all Blocks for a given course (i.e. on any for any framework which 
     * has been associated with the course UNION for any user which is enrolled in the 
     * course) which have at least one signup.
     * @param courseId A BB course_main pk
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of all Blocks for the given course which have at least one signup.
     */
    public static BlockList loadUsedByCourseId (int courseId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByCourseId (courseId, null, null);
    }

    /**
     * Gets a list of all Blocks for a given course  (i.e. on any for any framework which 
     * has been associated with the course UNION for any user which is enrolled in the 
     * course) which have at least one signup and which start no earlier than the specified 
     * start datetime and end no later than the specified end datetime.
     * @param courseId A BB course_main pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of all Blocks for the given course which have at least one signup and which start 
     * no earlier than the specified start datetime and end no later than the specified 
     * end datetime.
     */
    public static BlockList loadUsedByCourseId (int courseId, Date startDate, Date endDate)
         throws ConnectionNotAvailableException, InterruptedException, SQLException
   {        
        return loadUsedByUserIdOwnerIdAndCourseId (0, 0, courseId, startDate, endDate);
    }
    
    /**
     * Gets a list of Blocks for a given course  (i.e. on any for any framework which 
     * has been associated with the course UNION for any user which is enrolled in the 
     * course) for which a given user has signed up.
     * @param userId A BB user pk
     * @param courseId A BB course main pk
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of Blocks for a given course for which a given user has signed up.
     */
    public static BlockList loadUsedByUserIdAndCourseId (int userId, int courseId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByUserIdOwnerIdAndCourseId (userId, 0, courseId, null, null);
    }

    /**
     * Gets a list of Blocks for a given course (i.e. on any for any framework which 
     * has been associated with the course UNION for any user which is enrolled in the 
     * course) for which a given user has signed up and which start  no earlier than the 
     * specified start datetime and end no later than the specified end datetime.
     * @param userId A BB user pk
     * @param courseId A BB course main pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of Blocks for a given course for which a given user has signed up and which start  no earlier than the 
     * specified start datetime and end no later than the specified end datetime.
     */
    public static BlockList loadUsedByUserIdAndCourseId (int userId, int courseId, Date startDate, Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadUsedByUserIdOwnerIdAndCourseId (userId, 0, courseId, startDate, endDate);
    }

    /**
     * Gets a list of Blocks which are owned by a given user for a given course (i.e. on any for any framework which 
     * has been associated with the course UNION for any user which is enrolled in the 
     * course) for which another given user has signed up and which start  no earlier than 
     * the specified start datetime and end no later than the specified end datetime.
     * @param userId A BB user pk - the person signed up
     * @param ownerId A BB user pk - the owner of the blocks
     * @param courseId A BB course main pk
     * @param startDate A datetime - the earliest time possible of a Block in this list. If null then
     * there's no early limit.
     * @param endDate A datetime - the latest time possible of a Block in this list. If null then
     * there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a list of Blocks which are owned by a given user for a given course for 
     * which another given user has signed up and which start  no earlier than 
     * the specified start datetime and end no later than the specified end 
     * datetime.
     */
    public static BlockList loadUsedByUserIdOwnerIdAndCourseId (int userId, int ownerId, int courseId, Date startDate, Date endDate)
         throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        String sqlText = 
            "SELECT ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment, count(ssslot.ss_slot_id) as BLOCK_USAGE "+
            " FROM wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot, wms_ss_vw_framework_nd ssframework "+((courseId != 0)?", wms_ss_access ssaccess ":" ")+
            " WHERE  1=1 "+
            " AND ssframework.ss_framework_id = ssblock.ss_framework_id "+
            (ownerId==0?"":"AND (ssblock.owner_user_pk1 = "+Integer.toString(ownerId)+" OR ssframework.owner_user_pk1 = "+Integer.toString(ownerId)+") ") +                
            (userId==0?"":"AND ssslot.user_pk1 = " + Integer.toString(userId) + " ") +
            " AND ssblock.ss_block_id = ssslot.ss_block_id ";
        if (courseId != 0)
        { sqlText = sqlText +
            " AND ssaccess.ss_framework_id = ssframework.ss_framework_id "+
            " AND ssaccess.access_category = 'COURSE' "+
            " AND ssaccess.allowed_pk1 = " + Integer.toString(courseId) + " ";
        }
        sqlText = SqlUtil.appendDateRangeCheck(sqlText, "ssblock.start_datetime", "ssblock.end_datetime", startDate, endDate);
        sqlText = sqlText + " GROUP BY ssblock.ss_block_id, ssblock.date_created, ssblock.date_updated, ssblock.deleted, ssblock.owner_user_pk1, ssblock.ss_framework_id, ssblock.ss_block_set_id, ssblock.start_datetime, ssblock.end_datetime, ssblock.max_slots, ssblock.admin_comment";
        sqlText = sqlText + " ORDER BY ssblock.start_datetime";

        return BlockList.createFromSql(sqlText);
    }

    /** private static methods */
    private static BlockList createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        BlockList bl = new BlockList();
        while (res.next())
        {
            Block b = new Block(res);
            try
            {
                res.findColumn("BLOCK_USAGE");
                b.setUsedSlots(res.getInt("BLOCK_USAGE"));
            }
            catch (SQLException e)
            {
                //e.printStackTrace();
            }
            bl.add(b);
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        
        return bl;
    }
    
    /** object methods */

    /** constructors */
    private BlockList() {
    }
    
}
