/*
 * DataUtil.java
 *
 * Created on August 26, 2005, 9:35 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import blackboard.db.*;
import blackboard.persist.Id;
import wms.ss.db.*;

/**
 * This object contains a number of static utility functions that are handy when 
 * dealing with the various data objects.
 * @author cwarren
 */
public class DataUtil {
    
    /**
     * An index used to access the days part of the array returned by the dateDifference 
     * function.
     * @see dateDifference
     */
    public static int IDX_DAYS = 0;
    /**
     * An index used to access the hours part of the array returned by the dateDifference 
     * function.
     * @see dateDifference
     */
    public static int IDX_HOURS = 1;
    /**
     * An index used to access the minutes part of the array returned by the dateDifference 
     * function.
     * @see dateDifference
     */
    public static int IDX_MINUTES = 2;
    /**
     * An index used to access the seconds part of the array returned by the dateDifference 
     * function.
     * @see dateDifference
     */
    public static int IDX_SECONDS = 3;
    /**
     * An index used to access the milliseconds part of the array returned by the dateDifference 
     * function.
     * @see dateDifference
     */
    public static int IDX_MILLISECONDS = 4;

    /**
     * A constant - the number of milliseconds in 1 second
     */
    public static long MILLISECONDS_IN_SECOND = 1000;
    /**
     * A constant - the number of milliseconds in 1 minute
     */
    public static long MILLISECONDS_IN_MINUTE = MILLISECONDS_IN_SECOND * 60;
    /**
     * A constant - the number of milliseconds in 1 hour
     */
    public static long MILLISECONDS_IN_HOUR = MILLISECONDS_IN_MINUTE * 60;
    /**
     * A constant - the number of milliseconds in 1 day
     */
    public static long MILLISECONDS_IN_DAY = MILLISECONDS_IN_HOUR * 24;

    // #444488 - dark blue - distant 2 or more days
    // #448888 - mid blue - approaching 1-2 days
    // #448844 - green - nigh < 1 day
    // #888844 - yellow - imminent < 1 hour
    // #884444 - red - now < 10 minutes
    /**
     * A hex color code (#xxxxxx) used to indicate an event a far off in time
     */
    public static String TIMING_COLOR_DISTANT = "#444488";
    /**
     * A hex color code (#xxxxxx) used to indicate an event a near in time, but not today
     */
    public static String TIMING_COLOR_APPROACHING = "#448888";
    /**
     * A hex color code (#xxxxxx) used to indicate an event that will happen today
     */
    public static String TIMING_COLOR_TODAY = "#448844";
    /**
     * A hex color code (#xxxxxx) used to indicate an event that will happen very soon 
     * (on a scale of minutes or hours).
     */
    public static String TIMING_COLOR_IMMINENT = "#884444";
    /**
     * A hex color code (#xxxxxx) used to indicate an event that is about to 
     * start (< 5 minutes or so) or has already started
     */
    public static String TIMING_COLOR_NOW = "#CC4444";
    
    /**
     * Gets the integer primary key from a BB ID string of the form _xxx_xxx_
     * @param theId a BB ID as gotten from the getID function of the various BB api data objects
     * @return An integer that is the PK1 for the record associated with that object.
     */
    public static int getPkFromBbId(Id theId)
    {
        String pkStr = theId.toExternalString();
        return Integer.parseInt(pkStr.substring(1,pkStr.indexOf('_',1)));
    }

    /**
     * Given a userid (log on name), gets the PK for the DB record associated with that 
     * user. NOTE: does a DB call.
     * @param userIdStr The user ID (log on name)
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The BB pk for that user
     */
    public static int getUserPkFromUserId(String userIdStr)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT pk1 FROM users WHERE user_id = '"+userIdStr+"' AND row_status = 0";
        return SqlUtil.runSqlGetIntField(sqlText, "pk1");
    }
    
    /**
     * Given a course id (which for Williams looks like YYS-SUBJ-NUM-SN), gets the PK for the DB record associated with that 
     * course. NOTE: does a DB call.
     * @param courseIdStr A course ID (which for Williams looks like YYS-SUBJ-NUM-SN)
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The BB pk for that course
     */
    public static int getCoursePkFromCourseId(String courseIdStr)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT pk1 FROM course_main WHERE course_id = '"+courseIdStr+"' AND row_status = 0";
        return SqlUtil.runSqlGetIntField(sqlText, "pk1");
    }

    /**
     * Gets an array of course ID strings based on a substring/pattern.
     * @param courseIdSubstring A string that's a part of a course ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return An array of all course ids that have the given substring as part of 
     * their course id.
     */
    public static String [] getCourseIdListFromSubstring(String courseIdSubstring)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        
        String sqlText = "SELECT course_id FROM course_main WHERE UPPER(course_id) like '%"+courseIdSubstring.toUpperCase()+"%' AND row_status = 0";
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        ArrayList idList = new ArrayList(20);
        
        while (res.next())
        {
            idList.add(res.getString("course_id"));
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        String[] ret = new String[idList.size()];
        for (int i = 0; i < idList.size(); i++)
        {
            ret[i] = (String)(idList.get(i));
        }
        
        return ret; //(String[])(idList.toArray());
    }

    // returns the difference between the two dates in days, hours, minutes, seconds, and milliseconds
    /**
     * Returns an array of integers representing the days, hours, minutes, seconds, and 
     * milliseconds between the two given dates. The appropriate array indexes for accessing 
     * the various units are found in the IDX_* constants.
     * @param earlierDate The earlier date-time as a Calendar object
     * @param laterDate The later date time as a Calendar object
     * @return an array of 5 integers, representing the days, hours, minutes, seconds, and 
     * milliseconds between the two given dates.
     * @see IDX_DAYS
     * @see IDX_HOURS
     * @see IDX_MINUTES
     * @see IDX_SECONDS
     * @see IDX_MILLISECONDS
     */
    public static int[] dateDifference(java.util.Calendar earlierDate, java.util.Calendar laterDate)
    {
        return dateDifference(earlierDate.getTime(), laterDate.getTime());
    }
    
    // returns the difference between the two dates in days, hours, minutes, seconds, and milliseconds
    /**
     * Returns an array of integers representing the days, hours, minutes, seconds, and 
     * milliseconds between the two given dates. The appropriate array indexes for accessing 
     * the various units are found in the IDX_* constants.
     * @param earlierDate The earlier date-time as a Date object
     * @param laterDate The later date-time as a Date object
     * @return An array of integers representing the days, hours, minutes, seconds, and 
     * milliseconds between the two given dates.
     */
    public static int[] dateDifference(java.util.Date earlierDate, java.util.Date laterDate)
    {
        int[] returnValues = new int[5];
        
        long earlyMs = earlierDate.getTime();
        long lateMs = laterDate.getTime();
        
        long timeDiff = lateMs - earlyMs;
        returnValues[IDX_DAYS] = (int)(timeDiff / MILLISECONDS_IN_DAY);
        timeDiff = timeDiff % MILLISECONDS_IN_DAY;
        returnValues[IDX_HOURS] = (int)(timeDiff / MILLISECONDS_IN_HOUR);
        timeDiff = timeDiff % MILLISECONDS_IN_HOUR;
        returnValues[IDX_MINUTES] = (int)(timeDiff / MILLISECONDS_IN_MINUTE);
        timeDiff = timeDiff % MILLISECONDS_IN_MINUTE;
        returnValues[IDX_SECONDS] = (int)(timeDiff / MILLISECONDS_IN_SECOND);
        timeDiff = timeDiff % MILLISECONDS_IN_SECOND;
        returnValues[IDX_MILLISECONDS] = (int)(timeDiff);
        
        return returnValues;
    }

    // takes a time array as returned by dateDifference and return the appropriate string description
    /**
     * Generates a human-readable string that's a time duration from an array of integers 
     * representing days, hours, minutes, seconds, and milliseconds (as generated by 
     * dateDifference).
     * @param timeArray An array of integers representing days, hours, minutes, seconds, and 
     * milliseconds (as generated by dateDifference)
     * @return A human-readable string that's a time duration
     * @see dateDifference
     */
    public static String getTimingText (int[] timeArray)
    {
        String untilStr = "";
        if (timeArray[DataUtil.IDX_DAYS] > 1) 
        {
            untilStr = Integer.toString(timeArray[DataUtil.IDX_DAYS]) + " days ";
        } else if (timeArray[DataUtil.IDX_DAYS] == 1)
        {
            untilStr = "1 day ";
        }
        if (timeArray[DataUtil.IDX_HOURS] > 1) 
        {
            untilStr = untilStr + Integer.toString(timeArray[DataUtil.IDX_HOURS]) + " hours ";
        } else if (timeArray[DataUtil.IDX_HOURS] == 1)
        {
            untilStr = untilStr +  "1 hour ";
        }
        if (timeArray[DataUtil.IDX_MINUTES] > 1) 
        {
            untilStr = untilStr + Integer.toString(timeArray[DataUtil.IDX_MINUTES]) + " minutes ";
        } else if (timeArray[DataUtil.IDX_MINUTES] == 1)
        {
           untilStr = untilStr +  "1 minute ";
        }
        return untilStr;
    }

    // takes a time array as returned by dateDifference and return the appropriate string description for only the largest relevant units
    /**
     * Generates a human-readable string that's an approximate time duration (to the nearest, 
     * largest non-zero unit) from an array of integers representing days, hours, minutes, 
     * seconds, and milliseconds (as generated by dateDifference).
     * @param timeArray An array of integers representing days, hours, minutes, seconds, and 
     * milliseconds (as generated by dateDifference)
     * @return A human readable string for the non-zero unit represting the largest 
     * interval of time (e.g. 0, 2, 34, 12, 132 would generate '2 hours').
     * @see dateDifference
     */
    public static String getTimingShortText (int[] timeArray)
    {
        String untilStr = "";
        if (timeArray[DataUtil.IDX_DAYS] > 1) 
        {
            return Integer.toString(timeArray[DataUtil.IDX_DAYS]) + " days ";
        } else if (timeArray[DataUtil.IDX_DAYS] == 1)
        {
            return "2 days"; // i.e. 1 day + unspecified hours
        }        
        if (timeArray[DataUtil.IDX_HOURS] > 1) 
        {
            return Integer.toString(timeArray[DataUtil.IDX_HOURS]) + " hours ";
        } else if (timeArray[DataUtil.IDX_HOURS] == 1)
        {
            return "1 hour " + ((timeArray[DataUtil.IDX_MINUTES] > 1)?"":(Integer.toString(timeArray[DataUtil.IDX_MINUTES]) + " min "));
        }
        if (timeArray[DataUtil.IDX_MINUTES] > 3) 
        {
            return Integer.toString(timeArray[DataUtil.IDX_MINUTES]) + " minutes ";
        }
        return "NOW!";
    }

    // takes a time array as returned by dateDifference and return the appropriate timing color
    /**
     * Figures out an appropriate hexidecimal color (#xxxxxx) based on an array of integers 
     * representing days, hours, minutes, seconds, and milliseconds (as generated by 
     * dateDifference).
     * @param timeArray An array of integers representing days, hours, minutes, seconds, and 
     * milliseconds (as generated by dateDifference)
     * @return A hexidecimal color code (#xxxxxx)
     * @see dateDifference
     */
    public static String getTimingColor (int[] timeArray)
    {
        String timeColor = "";
        if (timeArray[DataUtil.IDX_DAYS] > 1) // distant 2 or more days
        {
            return DataUtil.TIMING_COLOR_DISTANT;
        } else if (timeArray[DataUtil.IDX_DAYS] == 1) // approaching 1-2 days
        {
            return DataUtil.TIMING_COLOR_APPROACHING;
        } else if (timeArray[DataUtil.IDX_HOURS] > 1) // today < 1 day
        {
            return DataUtil.TIMING_COLOR_TODAY;
        } else if ((timeArray[DataUtil.IDX_HOURS] <= 1) && (timeArray[DataUtil.IDX_MINUTES] > 10))
        { // imminent <= 1 hour
            return DataUtil.TIMING_COLOR_IMMINENT;
        } else { // now <= 10 minutes
            return DataUtil.TIMING_COLOR_NOW;
        }
    }

    /** Creates a new instance of DataUtil */
    private DataUtil() {
    }
    
}
