/*
 * Framework.java
 *
 * Created on August 18, 2005, 1:23 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.util.Calendar;
import java.util.Date;
import wms.ss.data.SlotSignup;
import wms.ss.data.Block;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;

/**
 * This represents the high level attributes of a signup sheet. It contains all the 
 * information that applies to the whole sheet (e.g. sheet name, description, limitations, 
 * etc) and has associated with it various time blocks for which people may sign up. It 
 * also has associated with it various access permissions that grant either individuals 
 * or classes access to sign up for any time block associated with this Framework.
 * 
 * A signup sheet consists of a Framework, zero or more Blocks, and zero or more Access 
 * records.
 * @author cwarren
 */
public class Framework {

    private int frameworkId;
    private java.util.Date dateCreated, dateUpdated, signupDeadline;
    private int ownerUserPk1 , groupId, maxTotalSlotsPerUser, maxPendingSlotsPerUser;
    private String name, description, resource, location, changeAlertOwnerFlag, imminentAlertOwnerFlag;

    private static int nameSizeLimit = 64;
    private static int descriptionSizeLimit = 255;
    private static int resourceSizeLimit = 64;
    private static int locationSizeLimit = 64;
          
   /**
     * This is one of the various loader methods that retrieves a Framework from the 
     * database. This one gets the Framework that contains the Block which has the 
     * SlotSignup of the given ID.
     * @param slotSignupId The ID of a SlotSignup
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The Framework that contains the Block which has the SlotSignup of the given ID.
     */
    public static Framework loadBySlotSignupId (int slotSignupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return Framework.createFromSql("SELECT ssframework.* "+
                "FROM wms_ss_vw_framework_nd ssframework, wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot "+
                "WHERE ssframework.ss_framework_id=ssblock.ss_framework_id "+
                " AND ssblock.ss_block_id=ssslot.ss_block_id " +
                "AND ssslot.ss_slot_id="+Integer.toString(slotSignupId));
    }

    /**
     * This is one of the various loader methods that retrieves a Framework from the 
     * database. This one gets the Framework that contains the Block of the given ID.
     * @param blockId A Block ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The Framework that contains the Block of the given ID.
     */
    public static Framework loadByBlockId (int blockId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return Framework.createFromSql("SELECT ssframework.* "+
                "FROM wms_ss_vw_framework_nd ssframework, wms_ss_vw_block_nd ssblock "+
                "WHERE ssframework.ss_framework_id=ssblock.ss_framework_id "+
                " AND ssblock.ss_block_id="+Integer.toString(blockId));
    }


    /**
     * This is one of the various loader methods that retrieves a Framework from the 
     * database. This one gets the Framework of the given ID.
     * @param frameworkId A Framework ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The Framework of the given ID.
     */
    public static Framework loadByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return Framework.createFromSql("SELECT ssframework.* "+
                "FROM wms_ss_vw_framework_nd ssframework " +
                "WHERE ssframework.ss_framework_id="+Integer.toString(frameworkId));
    }

    /**
     * Marks a Framework for deletion. Also calls the delete method for all associated Blocks, 
     * and marks all associated Access records for deletion.
     * @param frameworkId The ID of the Framework to be deleted
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        BlockList bl = BlockList.loadByFrameworkId(frameworkId);
        for (int bli = 0; bli < bl.size(); bli++)
        {
            Block.deleteByBlockId(((Block)bl.get(bli)).getBlockId(),"the signup sheet was deleted");
        }
        SqlUtil.runSql("UPDATE wms_ss_framework SET deleted = 'Y' WHERE ss_framework_id="+Integer.toString(frameworkId));
        SqlUtil.runSql("UPDATE wms_ss_access SET deleted = 'Y' WHERE ss_framework_id="+Integer.toString(frameworkId));
    }

    /** private static methods */
    private static Framework createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        
        Framework f = new Framework();
        if (res.next())
        {
            f = new Framework(res);
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return f;
    }    
    
    /** object methods */

    /**
     * Creates a new Framework object, with all fields uninitialized
     */
    public Framework() {
    }

    /**
     * Creates a new Framework object with the given initial values
     * @param ownerUserPk1 
     * @param groupId 
     * @param maxTotalSlotsPerUser 
     * @param maxPendingSlotsPerUser 
     * @param name 
     * @param description 
     * @param resource 
     * @param location 
     */
    public Framework(
            int ownerUserPk1, 
            int groupId, 
            int maxTotalSlotsPerUser, 
            int maxPendingSlotsPerUser,
            String name, 
            String description, 
            String resource, 
            String location)
    {
        this.frameworkId = -1; 
        this.dateCreated = null; 
        this.dateUpdated = null;
        this.ownerUserPk1 = ownerUserPk1; 
        this.groupId = groupId; 
        this.maxTotalSlotsPerUser = maxTotalSlotsPerUser; 
        this.maxPendingSlotsPerUser = maxPendingSlotsPerUser;
        
        this.setName(name);
        this.setDescription(description);
        this.setResource(resource); 
        this.setLocation(location);
    }

    /**
     * Creates a new Framework object with the given initial values
     * @param frameworkId 
     * @param dateCreated 
     * @param dateUpdated 
     * @param ownerUserPk1 
     * @param groupId 
     * @param maxTotalSlotsPerUser 
     * @param maxPendingSlotsPerUser 
     * @param name 
     * @param description 
     * @param resource 
     * @param location 
     */
    public Framework(int frameworkId, 
            java.util.Date dateCreated, 
            java.util.Date dateUpdated,
            int ownerUserPk1, 
            int groupId, 
            int maxTotalSlotsPerUser, 
            int maxPendingSlotsPerUser,
            String name, 
            String description, 
            String resource, 
            String location)
    {
        this.frameworkId = frameworkId; 
        this.dateCreated = dateCreated; 
        this.dateUpdated = dateUpdated;
        this.ownerUserPk1 = ownerUserPk1; 
        this.groupId = groupId; 
        this.maxTotalSlotsPerUser = maxTotalSlotsPerUser; 
        this.maxPendingSlotsPerUser = maxPendingSlotsPerUser;
        
        this.setName(name);
        this.setDescription(description);
        this.setResource(resource); 
        this.setLocation(location);
    }

    /**
     * Creates a new Framework object from a java.sql.ResultSet which has all the fields from 
     * a Framework DB record.
     * @param res 
     * @throws java.sql.SQLException 
     */
    public Framework(ResultSet res)
        throws SQLException
    {
        this.frameworkId = res.getInt("SS_FRAMEWORK_ID"); 
        this.dateCreated = res.getTimestamp("DATE_CREATED"); 
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setOwnerUserPk1(res.getInt("OWNER_USER_PK1")); 
        this.setGroupId(res.getInt("SS_GROUP_ID")); 
        this.setMaxTotalSlotsPerUser(res.getInt("MAX_TOTAL_SLOTS_PER_USER")); 
        this.setMaxPendingSlotsPerUser(res.getInt("MAX_PENDING_SLOTS_PER_USER"));
        this.setName(res.getString("NAME"));
        this.setDescription(res.getString("DESCRIPTION"));
        this.setChangeAlertOwnerFlag(res.getString("CHANGE_ALERT_OWNER_FLAG"));
        this.setImminentAlertOwnerFlag(res.getString("imminent_alert_owner_flag"));
        this.setSignupDeadline(res.getTimestamp("SIGNUP_DEADLINE"));
        this.setResource(res.getString("SS_RESOURCE")); 
        this.setLocation(res.getString("SS_LOCATION"));
    }

    /**
     * Updates the DB record for this Framework
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void save() 
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "UPDATE wms_ss_framework "+
            " SET "+
            " date_updated               = SYSDATE, "+
            " ss_group_id                = "+Integer.toString(this.getGroupId())+", "+
            " name                       = '"+this.getName().replaceAll("'", "''")+"', "+
            " description                = '"+this.getDescription().replaceAll("'", "''")+"', "+
            " max_total_slots_per_user   = "+Integer.toString(this.getMaxTotalSlotsPerUser())+", "+
            " max_pending_slots_per_user = "+Integer.toString(this.getMaxPendingSlotsPerUser())+", "+
            " change_alert_owner_flag    = '"+this.getChangeAlertOwnerFlag()+"', "+
            " imminent_alert_owner_flag  = '"+this.getImminentAlertOwnerFlag()+"', "+
            " signup_deadline            = "+((this.getSignupDeadline()==null)?"NULL,":"to_date('"+SqlUtil.dateFormatter.format(this.getSignupDeadline())+"','"+SqlUtil.dateFormatStringForDB+"'), ")+
            " ss_resource                = '"+this.getResource().replaceAll("'", "''")+"', "+
            " ss_location                = '"+this.getLocation().replaceAll("'", "''")+"' "+
            " WHERE  ss_framework_id            = "+Integer.toString(this.frameworkId);
        
        SqlUtil.runSql(sqlText);
    }

    /**
     * Creates a new DB record for this framework, setting the ID in the process.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew() 
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
        this.frameworkId = newId;
        Calendar cal = Calendar.getInstance();
        this.dateCreated = cal.getTime();
        this.dateUpdated = cal.getTime();
        String sqlText = 
                "INSERT INTO wms_ss_framework ("+
                   " ss_framework_id, date_created, date_updated, " +
                   " deleted, owner_user_pk1, ss_group_id, " +
                   " name, description, max_total_slots_per_user, max_pending_slots_per_user, " +
                    "change_alert_owner_flag, imminent_alert_owner_flag, signup_deadline, " +
                    "ss_resource, ss_location) " +
               " VALUES ("
                    +Integer.toString(this.frameworkId)+"," // ss_group_id
                    +"SYSDATE," // date_created
                    +"SYSDATE," // date_updated
                    +"'N'," // deleted
                    +Integer.toString(this.getOwnerUserPk1())+"," // owner_user_pk1
                    +Integer.toString(this.getGroupId())+"," //ss_group_id
                    +"'"+this.getName().replaceAll("'", "''")+"'," // name
                    +"'"+this.getDescription().replaceAll("'", "''")+"'," // description
                    +Integer.toString(this.getMaxTotalSlotsPerUser())+"," // max_total_slots_per_user
                    +Integer.toString(this.getMaxPendingSlotsPerUser())+"," // max_pending_slots_per_user
                    +"'"+this.getChangeAlertOwnerFlag()+"', "
                    +"'"+this.getImminentAlertOwnerFlag()+"', "
                    +((this.getSignupDeadline()==null)?"NULL,":"to_date('"+SqlUtil.dateFormatter.format(this.getSignupDeadline())+"','"+SqlUtil.dateFormatStringForDB+"'), ")
                    +"'"+this.getResource().replaceAll("'", "''")+"'," // ss_resource
                    +"'"+this.getLocation().replaceAll("'", "''")+"'" // ss_location
                +")";
        SqlUtil.runSql(sqlText);
    }

    /**
     * Gets the name of the owner of this Framework
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The last name of the owner of this Framework.
     */
    public String getOwnerName()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String lastName = SqlUtil.runSqlGetStringField("select lastname from users where pk1="+this.getOwnerUserPk1(), "lastname");
        if (lastName.length() < 1)
        {
            lastName = "unknown";
        }
        
        return lastName;
    }

    /**
     * Gets the total number of times the user with the given ID has signed up for any Block 
     * associated with this sheet.
     * @param userPk1 A BB user pk
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The total number of times the user with the given ID has signed up for any Block 
     * associated with this sheet.
     */
    public int usageTotalByUser (int userPk1)
         throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        return usageByUserByTimeRange(userPk1,null,null);
    }
    
    /**
     * Gets the total number of times the user with the given ID has signed up for any Block 
     * associated with this sheet and which Block starts no earlier than the given start 
     * date-time and ends no later than the given end date-time.
     * @param userPk1 A BB user pk
     * @param startDate A date-time - the earliest time a Block could start
     * @param endDate A date-time - the latest time a Block could end
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The number of times the users signed up for a Block associated with this 
     * Framework and within the allowed time range.
     */
    public int usageByUserByTimeRange (int userPk1, java.util.Date startDate, java.util.Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText =
            "SELECT ssframework.ss_framework_id, count(ssslot.ss_slot_id) as framework_usage " +
            " FROM wms_ss_vw_framework_nd ssframework, wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot " +
            " WHERE ssframework.ss_framework_id = ssblock.ss_framework_id " +
            "  AND ssblock.ss_block_id = ssslot.ss_block_id " +
            "  AND ssframework.ss_framework_id = "+Integer.toString(this.getFrameworkId())+" " +
            "  AND ssslot.user_pk1 = "+Integer.toString(userPk1)+" ";
        sqlText = SqlUtil.appendDateRangeCheck(sqlText, "ssblock.start_datetime", "ssblock.end_datetime", startDate, endDate);
        sqlText = sqlText + " GROUP BY ssframework.ss_framework_id ";

        return SqlUtil.runSqlGetIntField(sqlText, "framework_usage");
    }

    /**
     * Returns the total number of Blocks associated with this Framework
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return the total number of Blocks associated with this Framework
     */
    public int getNumberOfBlocks()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return getNumberOfBlocks(false);
    }
   
    /**
     * Returns the total number of Blocks associated with this Framework which have space 
     * left in them (according to the limits set on the Blocks).
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The total number of Blocks associated with this Framework which have space 
     * left in them
     */
    public int getNumberOfOpenBlocks()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return getNumberOfBlocks(true);
    }

    private int getNumberOfBlocks(boolean openOnlyFlag)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = "SELECT count(ss_block_id) AS block_count" +
                " FROM wms_ss_vw_block_use ssblock " +
                " WHERE ssblock.ss_framework_id = " + Integer.toString(this.getFrameworkId());
        if (openOnlyFlag)
        {
            sqlText = sqlText + " AND ((ssblock.max_slots = -1) " +
                    " OR (ssblock.max_slots > ssblock.block_usage)) ";
        }
        return SqlUtil.runSqlGetIntField(sqlText, "block_count");
    }

    /**
     * Get the ID of this Framework
     * @return The ID of this Framework
     */

    public int getFrameworkId() {
        return frameworkId;
    }

    /**
     * Get the date-time this Framework was created
     * @return The date-time this Framework was created
     */
    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Get the date-time this Framework was last changed (saved).
     * @return the date-time this Framework was last changed
     * @see save
     */
    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Set the date-time this Framework was last changed
     * @param dateUpdated the date-time this Framework was last changed
     */
    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    /**
     * Get the BB pk of the owner of this sheet
     * @return the BB pk of the owner of this sheet
     */
    public int getOwnerUserPk1() {
        return ownerUserPk1;
    }

    /**
     * Set the BB pk of the owner of this sheet
     * @param ownerUserPk1 the BB pk of the owner of this sheet
     */
    public void setOwnerUserPk1(int ownerUserPk1) {
        this.ownerUserPk1 = ownerUserPk1;
    }

    /**
     * Get the ID of the SheetGroup with which the Framework is associated
     * @return the ID of the SheetGroup with which the Framework is associated
     */
    public int getGroupId() {
        return groupId;
    }

    /**
     * Set the ID of the SheetGroup with which the Framework is associated
     * @param groupId the ID of the SheetGroup with which the Framework is associated
     */
    public void setGroupId(int groupId) {
        this.groupId = groupId;
    }

    /**
     * Get the maximum total signups per user; i.e. the maximum total number of times a user can 
     * sign up for all Blocks associated with this Framework.
     * @return the maximum total signups per user
     */
    public int getMaxTotalSlotsPerUser() {
        return maxTotalSlotsPerUser;
    }

    /**
     * Set the maximum total signups per user; i.e. the maximum total number of times a user can 
     * sign up for all Blocks associated with this Framework.
     * @param maxTotalSlotsPerUser the maximum total signups per user
     */
    public void setMaxTotalSlotsPerUser(int maxTotalSlotsPerUser) {
        this.maxTotalSlotsPerUser = maxTotalSlotsPerUser;
    }

    /**
     * Get the maximum future signups a user can have for all Blocks associated with 
     * this Framework. E.g. if a user has signed up for 5 Blocks all told and 3 of 
     * them have already happened, then their Pending/Future count would be 2.
     * @return the maximum future signups a user can have for all Blocks associated with 
     * this Framework
     */
    public int getMaxPendingSlotsPerUser() {
        return maxPendingSlotsPerUser;
    }

    /**
     * Set the maximum future signups a user can have for all Blocks associated with 
     * this Framework.
     * @param maxPendingSlotsPerUser the maximum future signups a user can have for all Blocks associated with 
     * this Framework
     */
    public void setMaxPendingSlotsPerUser(int maxPendingSlotsPerUser) {
        this.maxPendingSlotsPerUser = maxPendingSlotsPerUser;
    }

    /**
     * Get the name of the signup sheet which this Framework represents
     * @return The name of the signup sheet
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of the signup sheet which this Framework represents
     * @param name The name of the signup sheet
     */
    public void setName(String name) {
        if (name == null) { name=""; }
        if (name.length() > this.nameSizeLimit)
        {
            name = name.substring(0, this.nameSizeLimit);
        }
        this.name = name;
    }

    /**
     * Get the description of the signup sheet which this Framework represents
     * @return the description of the signup sheet which this Framework represents
     */
    public String getDescription() {
        return description;
    }

    /**
     * Set the description of the signup sheet which this Framework represents
     * @param description the description of the signup sheet which this Framework represents
     */
    public void setDescription(String description) {
        if (description == null) { description=""; }
        if (description.length() > this.descriptionSizeLimit)
        {
            description = description.substring(0, this.descriptionSizeLimit);
        }
        this.description = description;
    }

    /**
     * Get the resource of the signup sheet which this Framework represents. The
     * resource is the limited thig to which this sheet is controlling access - a room,
     * a person, space in a car, etc. NOTE: this may be empty!
     * @return the resource of the signup sheet which this Framework represents.
     */
    public String getResource() {
        return resource;
    }

    /**
     * Set the resource of the signup sheet which this Framework represents.
     * @param resource the resource of the signup sheet which this Framework represents.
     */
    public void setResource(String resource) {
        if (resource == null) { resource=""; }
         if (resource.length() > this.resourceSizeLimit)
        {
            resource = resource.substring(0, this.resourceSizeLimit);
        }
       this.resource = resource;
    }

    /**
     * Set the location of the signup sheet which this Framework represents. NOTE: this may be empty!
     * @return the location of the signup sheet which this Framework represents.
     */
    public String getLocation() {
        return location;
    }

    /**
     * Set the location of the signup sheet which this Framework represents.
     * @param location the location of the signup sheet which this Framework represents.
     */
    public void setLocation(String location) {
        if (location == null) { location=""; }
        if (location.length() > this.locationSizeLimit)
        {
            location = location.substring(0, this.locationSizeLimit);
        }
        this.location = location;
    }

    /**
     * Get the signup deadline of the signup sheet which this Framework represents.
     * When the current date has passed the signup deadline current dates should be
     * disallowed.
     * @return the signup deadline of the signup sheet which this Framework represents.
     */
    public java.util.Date getSignupDeadline() {
        return signupDeadline;
    }

    /**
     * Set the signup deadline of the signup sheet which this Framework represents.
     * @param signupDeadline the signup deadline of the signup sheet which this Framework represents.
     */
    public void setSignupDeadline(java.util.Date signupDeadline) {
        this.signupDeadline = signupDeadline;
    }

    /**
     * Get the change-alert-owner flag of the signup sheet which this Framework represents.
     * This is used to determine whether or not to send an alert email to the owner of
     * this framework whenever a user signs up or removes their signup from this
     * framework.
     * @return the change-alert-owner flag of the signup sheet which this Framework represents.
     */
    public String getChangeAlertOwnerFlag() {
        return changeAlertOwnerFlag;
    }

    /**
     * Set the change-alert-owner flag of the signup sheet which this Framework represents.
     * @param changeAlertOwnerFlag the change-alert-owner flag of the signup sheet which this Framework represents.
     */
    public void setChangeAlertOwnerFlag(String changeAlertOwnerFlag) {
        if ((changeAlertOwnerFlag == null) || (changeAlertOwnerFlag.length()<1) || 
                (changeAlertOwnerFlag.equalsIgnoreCase("N")) || (changeAlertOwnerFlag.equalsIgnoreCase("0")))
        {
            this.changeAlertOwnerFlag = "N";
        }  
        else
        {
            this.changeAlertOwnerFlag = "Y";
        }
    }

    /**
     * Get the imminent-alert-owner flag of the signup sheet which this Framework represents.
     * This is used to determine whether or not to send an alert email to the owner of
     * this framework when there are imminent signups (e.g. with in 36 hours of the
     * current datetime).
     * @return the imminent-alert-owner flag of the signup sheet which this Framework represents.
     */
    public String getImminentAlertOwnerFlag() {
        return imminentAlertOwnerFlag;
    }

    /**
     * Set the imminent-alert-owner flag of the signup sheet which this Framework represents.
     * @param imminentAlertOwnerFlag the imminent-alert-owner flag of the signup sheet which this Framework represents.
     */
    public void setImminentAlertOwnerFlag(String imminentAlertOwnerFlag) {
        this.imminentAlertOwnerFlag = imminentAlertOwnerFlag;
    }

    /**
     * This sends and email message to all users who have at least one signup on this
     * sheet.
     * @param subject The subject of the message to be sent
     * @param message The text of the message to be sent.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void sendEmailToSignees(String subject, String message)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        BlockList bl = BlockList.loadByFrameworkId(this.getFrameworkId());
        for (int i = 0; i < bl.size(); i++)
        {
            Block b = (Block)(bl.get(i));
            b.sendEmailToSignees(subject, message);
        }
    }
    
}
