/*
 * SheetGroup.java
 *
 * Created on August 18, 2005, 3:45 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.util.Calendar;
import java.util.Date;
import java.sql.*;
import wms.ss.data.SlotSignup;
import wms.ss.data.Block;
import wms.ss.data.Framework;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;


/**
 * A SheetGroup is a way to tie a bunch of Frameworks together for ease of
 * organization and to give some attributes relevant to all sheets in the group
 * combined. In addition to the expected get/sets and other utility methods
 * SheetGroup has a number of LoadByX static methods which allow one to
 * get the group from a more detailed object (framework, block, etc.).
 * 
 * SheetGroups are created by users and each one is tied to a single user. Every
 * person that creates a Framework has at least one SheetGroup.
 * @author cwarren
 */
public class SheetGroup {

    /**
     * The name initially given to the default SheetGroup that is automatically created
     * when a user creates their first Framework.
     */
    public static String defaultGroupName = "the default group";
    
    private int sheetGroupId = 0;
    private java.util.Date dateCreated, dateUpdated;
    private int ownerUserPk1, maxTotalSlotsPerUser, maxPendingSlotsPerUser;
    private String name, description;
    private boolean isDefault;

    private static int nameSizeLimit = 64;
    private static int descriptionSizeLimit = 255;

   /**
     * Gets the SheetGroup that contains the Framework that contains the Block that
     * contains the specified SlotSignup
     * @param slotSignupId A SlotSignup ID
     * @return the SheetGroup that contains the Framework that contains the Block that
     * contains the specified SlotSignup
     */
    public static SheetGroup loadBySlotSignupId (int slotSignupId)
    {
        return null;
    }

   /**
    * Gets the SheetGroup that contains the Framework that contains the specified Block
    * @param blockId A Block ID
    * @return the SheetGroup that contains the Framework that contains the specified Block
    */
   public static SheetGroup loadByBlockId (int blockId)
    {
        return null;
    }

    /**
     * the default SheetGroup (i.e. the one in which newly created Frameworks are
     * placed) for a specified user.
     * @param userPk1 A BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A SheetGroup
     */
    public static SheetGroup loadDefaultForOwnerId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return createFromSql("SELECT " +
            " ssgroup.ss_group_id, ssgroup.date_created, ssgroup.date_updated,  " +
            " ssgroup.owner_user_pk1, ssgroup.name,  " +
            " ssgroup.description, ssgroup.max_total_slots_per_user, ssgroup.max_pending_slots_per_user, " +
            " ssgroup.is_default " +
            " FROM wms_ss_vw_group_nd ssgroup " +
            " WHERE ssgroup.is_default = 'Y' " +
            " AND ssgroup.owner_user_pk1 = "+Integer.toString(userPk1));
    }

    /**
     * Gets the SheetGroup that contains the specified Framework
     * @param frameworkId A Framework ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return the SheetGroup that contains the specified Framework
     */
    public static SheetGroup loadByFrameworkId (int frameworkId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return createFromSql("SELECT " +
            " ssgroup.ss_group_id, ssgroup.date_created, ssgroup.date_updated,  " +
            " ssgroup.owner_user_pk1, ssgroup.name,  " +
            " ssgroup.description, ssgroup.max_total_slots_per_user, ssgroup.max_pending_slots_per_user, " +
            " ssgroup.is_default " +
            " FROM wms_ss_vw_framework_nd ssframework, wms_ss_vw_group_nd ssgroup " +
            " WHERE ssgroup.ss_group_id = ssframework.ss_group_id " +
            " AND ssframework.ss_framework_id = "+Integer.toString(frameworkId));
    }

    /**
     * Get the SheetGroup with the given ID
     * @param sheetGroupId A SheetGroup ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The SheetGroup with that ID
     */
    public static SheetGroup loadByGroupId (int sheetGroupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return createFromSql("SELECT " +
            " ssgroup.ss_group_id, ssgroup.date_created, ssgroup.date_updated,  " +
            " ssgroup.owner_user_pk1, ssgroup.name,  " +
            " ssgroup.description, ssgroup.max_total_slots_per_user, ssgroup.max_pending_slots_per_user, " +
            " ssgroup.is_default " +
            " FROM wms_ss_vw_group_nd ssgroup " +
            " WHERE ssgroup.ss_group_id = "+Integer.toString(sheetGroupId));
    }

    /**
     * Deletes (i.e. marks as deleted) a given SheetGroup
     * @param sheetGroupId The ID of the SheetGroup to delete
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteByGroupId (int sheetGroupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        // first set all sheets in that group to the default group
        int userPk1 = SqlUtil.runSqlGetIntField("SELECT owner_user_pk1 FROM wms_ss_vw_group_nd WHERE ss_group_id = "+Integer.toString(sheetGroupId), "owner_user_pk1");
        SheetGroup defaultSheetGroup = SheetGroup.loadDefaultForOwnerId(userPk1);
        FrameworkList fl = FrameworkList.loadByGroupId(sheetGroupId);
        for (int flIndex = 0; flIndex < fl.size(); flIndex++)
        {
            Framework f = (Framework)fl.get(flIndex);
            f.setGroupId(defaultSheetGroup.getGroupId());
            f.save();
        }
        // now that that's done, set the deleted flag on this group
        SqlUtil.runSql("UPDATE wms_ss_group SET deleted = 'Y' WHERE ss_group_id = "+Integer.toString(sheetGroupId));
    }

    /** private static methods */
    private static SheetGroup createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        
        SheetGroup g = new SheetGroup();
        if (res.next())
        {
            g = new SheetGroup(res);
        }
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return g;
    }    

    /** object methods */

    /**
     * Creates an empty, blank SheetGroup
     */
    public SheetGroup() {
    }

    /**
     * Creates a new SheetGroup object from a java.sql.ResultSet which has all the fields from 
     * a SheetGroup DB record.
     * @param res java.sql.ResultSet which has all the fields from 
     * a SheetGroup DB record.
     * @throws java.sql.SQLException 
     */
    public SheetGroup(ResultSet res)
        throws SQLException
    {
        this.sheetGroupId = res.getInt("SS_GROUP_ID");
        this.dateCreated = res.getTimestamp("DATE_CREATED"); 
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setOwnerUserPk1(res.getInt("OWNER_USER_PK1")); 
        this.setName(res.getString("NAME"));
        this.setDescription(res.getString("DESCRIPTION"));
        this.setMaxTotalSlotsPerUser(res.getInt("MAX_TOTAL_SLOTS_PER_USER")); 
        this.setMaxPendingSlotsPerUser(res.getInt("MAX_PENDING_SLOTS_PER_USER"));
        this.setIsDefault(res.getString("IS_DEFAULT").equalsIgnoreCase("Y"));
    }

    /**
     * Store this SheetGroup object in the DB
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void save()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "UPDATE wms_ss_group "+
            " SET "+
            " date_updated               = SYSDATE, "+
            " ss_group_id                = "+Integer.toString(this.getGroupId())+", "+
            " name                       = '"+this.getName().replaceAll("'", "''")+"', "+
            " description                = '"+this.getDescription().replaceAll("'", "''")+"', "+
            " max_total_slots_per_user   = "+Integer.toString(this.getMaxTotalSlotsPerUser())+", "+
            " max_pending_slots_per_user = "+Integer.toString(this.getMaxPendingSlotsPerUser())+
            " WHERE  ss_group_id            = "+Integer.toString(this.getGroupId());
        
        SqlUtil.runSql(sqlText);
    }

    /**
     * Save this SheetGroup as a new object - sets various important automaticly
     * managed things like the ID.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew() 
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
        this.sheetGroupId = newId;
        Calendar cal = Calendar.getInstance();
        this.dateCreated = cal.getTime();
        this.dateUpdated = cal.getTime();
        String sqlText = 
           "INSERT INTO wms_ss_group ("+
               " ss_group_id, date_created, date_updated, "+
               " deleted, owner_user_pk1, name, "+
               " description, max_total_slots_per_user, max_pending_slots_per_user," +
               " is_default) "+
           " VALUES ("
                +Integer.toString(this.sheetGroupId)+"," // ss_group_id
                +"SYSDATE," // date_created
                +"SYSDATE," // date_updated
                +"'N'," // deleted
                +Integer.toString(this.getOwnerUserPk1())+"," // owner_user_pk1
                +"'"+this.getName().replaceAll("'", "''")+"'," // name
                +"'"+this.getDescription().replaceAll("'", "''")+"'," // description
                +Integer.toString(this.getMaxTotalSlotsPerUser())+"," // max_total_slots_per_user
                +Integer.toString(this.getMaxPendingSlotsPerUser())+"," // max_pending_slots_per_user
                +(this.getIsDefault()?"'Y'":"'N'") // is_default
            +")";
        SqlUtil.runSql(sqlText);
    }

    /**
     * Gets the total number of times a given user has signed up for slots on signup sheets
     * in this SheetGroup
     * @param userPk1 A BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The total number of times a given user has signed up for slots on signup sheets
     * in this SheetGroup
     */
    public int usageTotalByUser (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        return usageByUserByTimeRange(userPk1,null,null);
    }
    
    /**
     * Gets the total number of times a given user has signed up for slots in the
     * given time range on signup sheets in this SheetGroup
     * @param userPk1 A BB user PK
     * @param startDate A datetime - the earliest datetime at which a signup is counted
     * @param endDate A datetime - the latest datetime at which a signup is counted
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return the total number of times a given user has signed up for slots in the
     * given time range on signup sheets in this SheetGroup
     */
    public int usageByUserByTimeRange (int userPk1, java.util.Date startDate, java.util.Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText =
            "SELECT ssgroup.ss_group_id, count(ssslot.ss_slot_id) as group_usage " +
            " FROM wms_ss_vw_group_nd ssgroup, wms_ss_vw_framework_nd ssframework, wms_ss_vw_block_nd ssblock, wms_ss_vw_slot_nd ssslot " +
            " WHERE ssgroup.ss_group_id = ssframework.ss_group_id " +
            "  AND ssframework.ss_framework_id = ssblock.ss_framework_id " +
            "  AND ssblock.ss_block_id = ssslot.ss_block_id " +
            "  AND ssgroup.ss_group_id = "+Integer.toString(this.getGroupId())+" " +
            "  AND ssslot.user_pk1 = "+Integer.toString(userPk1)+" ";
        sqlText = SqlUtil.appendDateRangeCheck(sqlText, "ssblock.start_datetime", "ssblock.end_datetime", startDate, endDate);
        sqlText = sqlText + " GROUP BY ssgroup.ss_group_id ";

        return SqlUtil.runSqlGetIntField(sqlText, "GROUP_USAGE");
    }    
    
    /**
     * Gets the ID of this SheetGroup
     * @return A SheetGroup ID
     * @deprecated use getSheetGoupId instead
     */

    public int getGroupId() {
        return getSheetGroupId();
    }

    /**
     * Gets the ID of this SheetGroup
     * @return the ID of this SheetGroup
     */
    public int getSheetGroupId() {
        return sheetGroupId;
    }

    /**
     * Gets the datetime this SheetGroup was created
     * @return the datetime this SheetGroup was created
     */
    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Gets the datetime this SheetGroup was last changed
     * @return the datetime this SheetGroup was last changed
     */
    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Sets the datetime this SheetGroup was last changed
     * @param dateUpdated the datetime this SheetGroup was last changed
     */
    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    /**
     * Gets the BB user PK of the owner of this
     * @return the BB user PK of the owner of this
     */
    public int getOwnerUserPk1() {
        return ownerUserPk1;
    }

    /**
     * Sets the BB user PK of the owner of this
     * @param ownerUserPk1 the BB user PK of the owner of this
     */
    public void setOwnerUserPk1(int ownerUserPk1) {
        this.ownerUserPk1 = ownerUserPk1;
    }

    /**
     * Get the maximum total number of slots a user can sign up for on all sheets in
     * this group combined.
     * @return the maximum total number of slots a user can sign up for on all sheets in
     * this group combined.
     */
    public int getMaxTotalSlotsPerUser() {
        return maxTotalSlotsPerUser;
    }

    /**
     * Set the maximum total number of slots a user can sign up for on all sheets in
     * this group combined.
     * @param maxTotalSlotsPerUser the maximum total number of slots a user can sign up for on all sheets in
     * this group combined.
     */
    public void setMaxTotalSlotsPerUser(int maxTotalSlotsPerUser) {
        this.maxTotalSlotsPerUser = maxTotalSlotsPerUser;
    }

    /**
     * Gets the maximum total number of slots whose time has not yet passed that a user
     * can sign up for on all sheets in this group combined.
     * @return the maximum total number of slots whose time has not yet passed that a user
     * can sign up for on all sheets in this group combined.
     */
    public int getMaxPendingSlotsPerUser() {
        return maxPendingSlotsPerUser;
    }

    /**
     * Sets the maximum total number of slots whose time has not yet passed that a user
     * can sign up for on all sheets in this group combined.
     * @param maxPendingSlotsPerUser the maximum total number of slots whose time has not yet passed that a user
     * can sign up for on all sheets in this group combined.
     */
    public void setMaxPendingSlotsPerUser(int maxPendingSlotsPerUser) {
        this.maxPendingSlotsPerUser = maxPendingSlotsPerUser;
    }

    /**
     * Get the name of this SheetGroup
     * @return the name of this SheetGroup
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of this SheetGroup
     * @param name the name of this SheetGroup
     */
    public void setName(String name) {
       if (name == null) { name=""; }
        if (name.length() > this.nameSizeLimit)
        {
            name = name.substring(0, this.nameSizeLimit);
        }
       this.name = name;
    }

    /**
     * Get the description of this SheetGroup
     * @return the description of this SheetGroup
     */
    public String getDescription() {
        return description;
    }

    /**
     * Set the description of this SheetGroup
     * @param description the description of this SheetGroup
     */
    public void setDescription(String description) {
       if (description == null) { description=""; }
         if (description.length() > this.descriptionSizeLimit)
        {
            description = description.substring(0, this.descriptionSizeLimit);
        }
       this.description = description;
    }

    /**
     * Gets a flag indicating whether or not this SheetGroup is a default group
     * @return a flag indicating whether or not this SheetGroup is a default group
     */
    public boolean getIsDefault() {
        return isDefault;
    }

    /**
     * Set a flag indicating whether or not this SheetGroup is a default group
     * @param isDefault a flag indicating whether or not this SheetGroup is a default group
     */
    public void setIsDefault(boolean isDefault) {
        this.isDefault = isDefault;
    }

    
}
