/*
 * SlotSignup.java
 *
 * Created on August 18, 2005, 11:48 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.sql.*;
import java.util.Date;
import java.util.Calendar;
import javax.mail.*;
import javax.mail.internet.*;
import wms.ss.data.Block;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import wms.ss.display.HtmlCalendarTool;
import wms.email.WmsEmailer;

import blackboard.db.*;

/**
 * This represents a single sign up that a user has made for a given time block on
 * a given framework. In addition to the standard constructors, get/sets, and other
 * utility functions, this has a few loadByX static methods to get a SlotSignup
 * that conforms to particular guidelines.
 * @author cwarren
 */
public class SlotSignup {

    private int slotSignupId;
    private java.util.Date dateCreated, dateUpdated;
    private int updatedByUserPk1, blockId, userPk1;
    private String userId,userFirstName,userLastName,userEmail,adminComment;

    private String frameworkName, frameworkAlertOwnerFlag, blockTiming, blockDate, ownerName, ownerEmail;
    
    /**
     * Get the SlotSignup with a given ID
     * @param slotSignupId the SlotSignup ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return a SlotSignup object
     */
    public static SlotSignup loadBySlotSignupId (int slotSignupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return SlotSignup.createFromSql("SELECT "+ 
                "ssslot.ss_slot_id, ssslot.date_created, ssslot.date_updated, " + 
                "ssslot.updated_by_user_pk1, ssslot.ss_block_id, ssslot.user_pk1, ssslot.admin_comment, " + 
                "u.user_id, u.firstname, u.lastname, u.email " + 
                " FROM wms_ss_vw_slot_nd ssslot, users u " +
                " WHERE u.row_status = 0 " + 
                " AND u.pk1 = ssslot.user_pk1 " + 
                " AND ssslot.ss_slot_id="+Integer.toString(slotSignupId));
    }

    /**
     * Gets the SlotSignup for a given user in a given block.
     * @param userPk1 A BB user PK
     * @param blockId A Block ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return A SlotSignup object
     */
    public static SlotSignup loadByUserIdBlockId (int userPk1, int blockId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return SlotSignup.createFromSql("SELECT "+ 
                "ssslot.ss_slot_id, ssslot.date_created, ssslot.date_updated, " + 
                "ssslot.updated_by_user_pk1, ssslot.ss_block_id, ssslot.user_pk1, ssslot.admin_comment, " + 
                "u.user_id, u.firstname, u.lastname, u.email " + 
                " FROM wms_ss_vw_slot_nd ssslot, users u " +
                " WHERE ssslot.ss_block_id="+Integer.toString(blockId) +
                " AND u.row_status = 0 " + 
                " AND u.pk1 = ssslot.user_pk1 " + 
                " AND ssslot.user_pk1="+Integer.toString(userPk1));
    }

    /**
     * Delete (i.e. mrk as deleted) the SlotSignup with the given ID.
     * @param slotSignupId A SlotSignup ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void deleteBySlotSignupId (int slotSignupId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        SqlUtil.runSql("UPDATE wms_ss_slot SET deleted='Y' WHERE ss_slot_id = "+Integer.toString(slotSignupId));
    }

    /** private static methods */
    private static SlotSignup createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        
        SlotSignup s = new SlotSignup();
        if (res.next())
        {
            s = new SlotSignup(res);
        }
        sql.close();            
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return s;
    }    
    
    /** object methods */

    /**
     * Creates and empty SlotSignup
     */
    public SlotSignup() {
    }

    /**
     * Create a SlotSignup from a ResultSet that has 1 row containing all the
     * fields from the SlotSignup table.
     * @param res a ResultSet
     * @throws java.sql.SQLException 
     */
    public SlotSignup(ResultSet res) 
        throws SQLException
    {
        this.slotSignupId = res.getInt("SS_SLOT_ID");
        this.dateCreated = res.getTimestamp("DATE_CREATED");
        this.setDateUpdated(res.getTimestamp("DATE_UPDATED"));
        this.setUpdatedByUserPk1(res.getInt("UPDATED_BY_USER_PK1"));
        this.setBlockId(res.getInt("SS_BLOCK_ID"));
        this.setUserPk1(res.getInt("USER_PK1"));
        this.setAdminComment(res.getString("ADMIN_COMMENT"));
        
        try
        {
            res.findColumn("USER_ID");
            this.setUserId(res.getString("USER_ID"));
        }
        catch (SQLException e)
        {
            this.setUserId("");
        }
        try
        {
            res.findColumn("FIRSTNAME");
            this.setUserFirstName(res.getString("FIRSTNAME"));
        }
        catch (SQLException e)
        {
            this.setUserFirstName("");
        }
        try
        {
            res.findColumn("LASTNAME");
            this.setUserLastName(res.getString("LASTNAME"));
        }
        catch (SQLException e)
        {
            this.setUserLastName("");
        }
        try
        {
            res.findColumn("EMAIL");
            this.setUserEmail(res.getString("EMAIL"));
        }
        catch (SQLException e)
        {
            this.setUserEmail("");
        }
    }

    /**
     * Store this to the DB
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void save()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
            String sqlText = "UPDATE wms_ss_slot " +
                    "SET date_updated=SYSDATE," +
                    "updated_by_user_pk1="+Integer.toString(this.getUpdatedByUserPk1())+"," +
                    "admin_comment='" +this.getAdminComment().replaceAll("'", "''")+ "'" +
                    " WHERE" +
                    " ss_slot_id="+Integer.toString(this.getSlotSignupId());
            SqlUtil.runSql(sqlText);        
    }

    /**
     * Store this to the DB for the first time. This also sets various important
     * automatically managed data, like the SlotSignup ID. IMPORTANT: the 
     * SlotSignup ID is set to -1 if the user already has a signup on that 
     * block!
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void saveNew()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        int existingSlotId = SqlUtil.runSqlGetIntField("SELECT ss_slot_id FROM wms_ss_vw_slot_nd WHERE ss_block_id = "+Integer.toString(this.getBlockId())+" AND user_pk1="+Integer.toString(this.getUserPk1()), "ss_slot_id");
        if (existingSlotId == 0)
        {
            int newId = SqlUtil.runSqlGetIntField("SELECT wms_ss_ids.nextval AS new_id FROM dual", "new_id");
            this.slotSignupId = newId;
            Calendar cal = Calendar.getInstance();
            this.dateCreated = cal.getTime();
            this.dateUpdated = cal.getTime();
            if (this.getAdminComment() == null)
            {
                this.setAdminComment(" ");
            }
            String sqlText = "INSERT INTO wms_ss_slot ("+
                        " ss_slot_id, date_created, date_updated, " +
                        " deleted, updated_by_user_pk1, ss_block_id, " +
                        " user_pk1,admin_comment) " +
                   " VALUES ("
                        +Integer.toString(this.getSlotSignupId())+"," // ss_block_id
                        +"SYSDATE," // date_created
                        +"SYSDATE," // date_updated
                        +"'N'," // deleted
                        +Integer.toString(this.getUpdatedByUserPk1())+"," // updated_by_user_pk1
                        +Integer.toString(this.getBlockId())+"," //ss_block_id
                        +Integer.toString(this.getUserPk1())+"," // user_pk1
                        +"'" +this.getAdminComment().replaceAll("'", "''")+ "'" // admin_comment
                    +")";
            SqlUtil.runSql(sqlText);
        } else {
            this.slotSignupId = -1; // signals a collision
        }
    }
    
    /**
     * Gets the email address of the user in this SlotSignup
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return the email address of the user in this SlotSignup
     */
    public String findUserEmail()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        if ((getUserEmail() == null) || (getUserEmail().length() < 1))
        {
            String sqlText = "SELECT email FROM users WHERE pk1="+Integer.toString(this.getUserPk1());
            setUserEmail(SqlUtil.runSqlGetStringField(sqlText, "EMAIL"));
        }
        return getUserEmail();
    }
    
    /**
     * Gets the ID of this SlotSignup
     * @return the ID of this SlotSignup
     */

    public int getSlotSignupId() {
        return slotSignupId;
    }

    /**
     * Gets the datetime this SlotSignup was created
     * @return the datetime this SlotSignup was created
     */
    public java.util.Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Gets the datetime this SlotSignup was last modified
     * @return  the datetime this SlotSignup was last modified
     */
    public java.util.Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Sets the datetime this SlotSignup was last modified
     * @param dateUpdated the datetime this SlotSignup was last modified
     */
    public void setDateUpdated(java.util.Date dateUpdated) {
        this.dateUpdated = dateUpdated;
    }

    /**
     * Gets the BB PK of the user that last modifed this SlotSignup
     * @return the BB PK of the user that last modifed this SlotSignup
     */
    public int getUpdatedByUserPk1() {
        return updatedByUserPk1;
    }

    /**
     * Set the BB PK of the user that last modifed this SlotSignup
     * @param updatedByUserPk1 the BB PK of the user that last modifed this SlotSignup
     */
    public void setUpdatedByUserPk1(int updatedByUserPk1) {
        this.updatedByUserPk1 = updatedByUserPk1;
    }

    /**
     * Get the ID of the Block that contains this SlotSignup
     * @return the ID of the Block that contains this SlotSignup
     */
    public int getBlockId() {
        return blockId;
    }

    /**
     * Sets the ID of the Block that contains this SlotSignup
     * @param blockId the ID of the Block that contains this SlotSignup
     */
    public void setBlockId(int blockId) {
        this.blockId = blockId;
    }

    /**
     * Get the BB user PK of the user that's signed up for a Block
     * @return the BB user PK of the user that's signed up for a Block
     */
    public int getUserPk1() {
        return userPk1;
    }

    /**
     * Set the BB user PK of the user that's signed up for a Block
     * @param userPk1 the BB user PK of the user that's signed up for a Block
     */
    public void setUserPk1(int userPk1) {
        this.userPk1 = userPk1;
    }

    /**
     * Get the ID (NOT the PK) of the user that's signed up for a Block
     * @return the ID (NOT the PK) of the user that's signed up for a Block
     */
    public String getUserId() {
        return userId;
    }

    /**
     * Set the ID (NOT the PK) of the user that's signed up for a Block
     * @param userId the ID (NOT the PK) of the user that's signed up for a Block
     */
    public void setUserId(String userId) {
        this.userId = userId;
    }

    /**
     * Get the first name of the user that's signed up for a Block
     * @return the first name of the user that's signed up for a Block
     */
    public String getUserFirstName() {
        return userFirstName;
    }

    /**
     * Set the first name of the user that's signed up for a Block. NOTE: this does not
     * affect the user info stored in the DB, only the name kept in memory in this
     * object.
     * @param userFirstName the first name of the user that's signed up for a Block
     */
    public void setUserFirstName(String userFirstName) {
        this.userFirstName = userFirstName;
    }

    /**
     * Get the last name of the user that's signed up for a Block
     * @return the last name of the user that's signed up for a Block
     */
    public String getUserLastName() {
        return userLastName;
    }

    /**
     * Set the last name of the user that's signed up for a Block. NOTE: this does not
     * affect the user info stored in the DB, only the name kept in memory in this
     * object.
     * @param userLastName the last name of the user that's signed up for a Block
     */
    public void setUserLastName(String userLastName) {
        this.userLastName = userLastName;
    }

    /**
     * Get the email of the user that's signed up for a Block
     * @return the email of the user that's signed up for a Block
     */
    public String getUserEmail() {
        return userEmail;
    }

    /**
     * Set the email of the user that's signed up for a Block. NOTE: this does not
     * affect the user info stored in the DB, only the name kept in memory in this
     * object.
     * @param userEmail the email of the user that's signed up for a Block.
     */
    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }

    /**
     * Send an email the user that's signed up for a Block alerting them that
     * they've been signed up and by who.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailAddAlertToUser()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.setBlockAndFrameworkInfo();
        String emailMessage = this.ownerName + " has signed you up on the "+ this.frameworkName+ " signup sheet for the " +this.blockTiming+ " time block on " + this.blockDate+".";
        sendTheEmail(WmsEmailer.alertsFromEmailAddress,
                this.findUserEmail(),
                    "New "+this.frameworkName+" signup",
                    emailMessage);
    }

    /**
     * Send an email the user that was signed up for a Block alerting them that
     * they've removed and by who.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailRemoveAlertToUser()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.emailRemoveAlertToUser("");
    }

    /**
     * Send an email the user that was signed up for a Block alerting them that
     * they've removed and by who.
     * @param removalReason The reason that the person was removed - included in the email.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailRemoveAlertToUser(String removalReason)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.setBlockAndFrameworkInfo();
        //String emailMessage = this.ownerName + " has removed your "+this.blockDate+", "+this.blockTiming+" sign up from the "+ this.frameworkName+ " signup sheet";
        String emailMessage = "Your "+this.blockDate+", "+this.blockTiming+" sign up has been removed from the "+ this.frameworkName+ " signup sheet";
        if ((removalReason != null) && (removalReason.length() > 0))
        {
            emailMessage += " because " + removalReason;
        }
        emailMessage += ".";
        sendTheEmail(WmsEmailer.alertsFromEmailAddress,
                this.findUserEmail(),
                    "Removed "+this.frameworkName+" signup",
                    emailMessage);
    }

    /**
     * Send an email the owner of a Block alerting them that a user has signed up for
     * it - includes the name of the user that signed up.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailAddAlertToBlockOwner()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.setBlockAndFrameworkInfo();
        if (this.frameworkAlertOwnerFlag.equals("Y"))
        {
            String emailMessage = this.getUserFirstName()+" "+this.getUserLastName()+" has signed up on "+ this.frameworkName+ " for "+this.blockDate+", "+this.blockTiming+".";
            sendTheEmail(WmsEmailer.alertsFromEmailAddress,
                    this.ownerEmail,
                        "New "+this.frameworkName+" signup",
                        emailMessage);
        }
    }

    /**
     * Send an email the owner of a Block alerting them that a user has been removed
     * from it - includes the name of the user that was removed.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailRemoveAlertToBlockOwner()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.setBlockAndFrameworkInfo();
        if (this.frameworkAlertOwnerFlag.equals("Y"))
        {
            String emailMessage = this.getUserFirstName()+" "+this.getUserLastName()+" has removed his/her "+this.blockDate+", "+this.blockTiming+" sign up from the "+ this.frameworkName+ " signup sheet.";
            sendTheEmail(WmsEmailer.alertsFromEmailAddress,
                    this.ownerEmail,
                        "Removed "+this.frameworkName+" signup",
                        emailMessage);
        }
    }

    /**
     * Send a message to the person signed up for a Block from the owner of that Block
     * - i.e. the Block owner's email address shows up in the From field.
     * @param subject The subject of the message
     * @param message The text of the message
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public void emailMessageFromBlockOwnerToUser(String subject, String message)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        this.setBlockAndFrameworkInfo();        
        sendTheEmail(this.ownerEmail,
                    this.findUserEmail(),
                    subject,
                    message);
    }
    
    private void sendTheEmail(String fromAddr, String toAddr, String subject, String msg)
    {
            WmsEmailer emailer = new WmsEmailer();
            try
            {
                emailer.sendMessage(fromAddr,
                                    toAddr,
                                    subject,
                                    msg);
            }
            catch (Exception e)
            {
            }
    }
    
    private void setBlockAndFrameworkInfo()
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        if ((this.blockTiming == null) || (this.blockDate == null) || (this.ownerName == null))
        {
            Block curBlock = Block.loadBySlotSignupId(this.getSlotSignupId());
            this.ownerName = SqlUtil.runSqlGetStringField("SELECT firstname || ' ' || lastname as user_full_name FROM users WHERE pk1="+Integer.toString(curBlock.getOwnerUserPk1()), "user_full_name");
            this.ownerEmail = SqlUtil.runSqlGetStringField("SELECT email FROM users WHERE pk1="+Integer.toString(curBlock.getOwnerUserPk1()), "email");
            Date d = curBlock.getStartDatetime();
            String s1 = HtmlCalendarTool.dayOfWeekDateFormatter.format(d);
            String s2 = HtmlCalendarTool.downToDayDateFormatter.format(d);
            this.blockDate =  s1 + ", " + s2;
            this.blockTiming = HtmlCalendarTool.hourMinAmPmFormatter.format(curBlock.getStartDatetime()) + "-" + 
                    HtmlCalendarTool.hourMinAmPmFormatter.format(curBlock.getEndDatetime());
        }
        if ((this.frameworkName == null) || (this.frameworkAlertOwnerFlag == null))
        {
            Framework curFramework = Framework.loadBySlotSignupId(this.getSlotSignupId());
            this.frameworkName = curFramework.getName();
            this.frameworkAlertOwnerFlag = curFramework.getChangeAlertOwnerFlag();
        }
   }

    public String getAdminComment() {
        return adminComment;
    }

    public void setAdminComment(String comment) {
        this.adminComment = comment;
    }
}
