/*
 * SlotSignupList.java
 *
 * Created on August 19, 2005, 9:43 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.data;

import java.util.ArrayList;
import java.util.Date;
import java.text.SimpleDateFormat;
import wms.ss.db.Connect;
import wms.ss.db.SqlUtil;
import blackboard.db.*;
import java.sql.*;

/**
 * This manages a list of related SlotSignups. SlotSignupLists are created via the
 * loadByX static methods rather than the constructor (which is private).
 * @author cwarren
 */
public class SlotSignupList  extends java.util.ArrayList {
    
   /**
     * Gets all SlotSignups that a given user has (i.e. has used to sign up for a
     * Block).
     * @param userPk1 A BB user PK
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return all SlotSignups that the given user has
     */
    public static SlotSignupList loadByUserId (int userPk1)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        return loadByUserId (userPk1, null, null);
    }
            
    /**
     * Gets all SlotSignups that a given user has and which fall within the given
     * datetime range.
     * @param userPk1 A BB user PK
     * @param startDate A datetime - the earliest time possible of a SlotSignup in this list. If null
     * then there's no early limit.
     * @param endDate A datetime - the latest time possible of a SlotSignup in this list. If null
     * then there's no late limit.
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return all the SlotSignups that the given user has and which fall within the given
     * datetime range.
     */
    public static SlotSignupList loadByUserId (int userPk1, Date startDate, Date endDate)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "SELECT " + 
            "ssslot.ss_slot_id, ssslot.date_created, ssslot.date_updated, " + 
            "ssslot.updated_by_user_pk1, ssslot.ss_block_id, ssslot.user_pk1, ssslot.admin_comment, " + 
            "u.user_id, u.firstname, u.lastname " + 
            " FROM wms_ss_vw_slot_nd ssslot, users u, wms_ss_vw_block_nd ssblock " + 
            " WHERE u.row_status = 0 " + 
            " AND u.pk1 = ssslot.user_pk1 " + 
            " AND ssslot.user_pk1 = " + Integer.toString(userPk1) +
            " AND ssblock.ss_block_id = ssslot.ss_block_id ";
        sqlText = SqlUtil.appendDateRangeCheck(sqlText, "ssblock.start_datetime", "ssblock.end_datetime", startDate, endDate);

        return createFromSql(sqlText);
    }

    /**
     * Gets all SlotSignups that a given user owns (i.e. not the ones that the user
     * has used to sign up Blocks, but the ones that are signed up for a Block that the
     * user owns).
     * UNIMPLEMENTED
     * @param userPk1 A BB user PK
     * @return all SlotSignups that a given user owns
     */
    public static SlotSignupList loadByOwnerUserId (int userPk1)
    {
        return loadByOwnerUserId (userPk1, null, null);
    }

    /**
     * Gets all SlotSignups in a date range that a given user owns (i.e. not the ones
     * that the user has used to sign up Blocks, but the ones that are signed up for a 
     * Block that the user owns).
     * UNIMPLEMENTED
     * @param userPk1 A BB user PK
     * @param startDate A datetime - the earliest time possible of a SlotSignup in this list. If null
     * then there's no early limit.
     * @param endDate A datetime - the latest time possible of a SlotSignup in this list. If null
     * then there's no late limit.
     * @return all SlotSignups in a date range that a given user owns
     */
    public static SlotSignupList loadByOwnerUserId (int userPk1, Date startDate, Date endDate)
    {
        SlotSignupList ssl = new SlotSignupList();

        return ssl;
    }

    /**
     * Get all SlotSignups in for Blocks on Framework in the given SheetGroup
     * UNIMPLEMENTED
     * @param groupId A SheetGroup ID
     * @return all SlotSignups in for Blocks on Sheets in the given SheetGroup
     */
    public static SlotSignupList loadByGroupId (int groupId)
    {
        return loadByGroupId (groupId, null, null);
    }

   /**
    * all SlotSignups in for Blocks within the given time range on Framework in the given
    * SheetGroup
    * UNIMPLEMENTED
    * @param groupId A SheetGroup ID
    * @param startDate A datetime - the earliest time possible of a SlotSignup in this list. If null
    * then there's no early limit.
    * @param endDate A datetime - the latest time possible of a SlotSignup in this list. If null
    * then there's no late limit.
    * @return all SlotSignups in for Blocks within the given time range on Sheets in the given
    * SheetGroup
    */
   public static SlotSignupList loadByGroupId (int groupId, Date startDate, Date endDate)
    {
        SlotSignupList ssl = new SlotSignupList();

        return ssl;
    }

    /**
     * all SlotSignups in for Blocks on the given Framework
     * UNIMPLEMENTED
     * @param frameworkId A Framework ID
     * @return all SlotSignups in for Blocks on the given Framework
     */
    public static SlotSignupList loadByFrameworkId (int frameworkId)
    {
        return loadByFrameworkId (frameworkId, null, null);
    }

    /**
     * all SlotSignups in for Blocks within the given time range on the given Sheet
     * UNIMPLEMENTED
     * @param frameworkId A Framework ID
     * @param startDate A datetime - the earliest time possible of a SlotSignup in this list. If null
     * then there's no early limit.
     * @param endDate A datetime - the latest time possible of a SlotSignup in this list. If null
     * then there's no late limit.
     * @return all SlotSignups in for Blocks within the given time range on the given Sheet
     */
    public static SlotSignupList loadByFrameworkId (int frameworkId, Date startDate, Date endDate)
    {
        SlotSignupList ssl = new SlotSignupList();

        return ssl;
    }

    /**
     * All SlotSignups for Blocks on Frameworks that a given Course has Access.
     * UNIMPLEMENTED
     * @param coursePk A BB course PK
     * @return All SlotSignups for Blocks on Frameworks that a given Course has Access
     */
    public static SlotSignupList loadByCourseId (int coursePk)
    {
        return loadByCourseId (coursePk, null, null);
    }

    /**
     * All SlotSignups for Blocks within the given time range on Frameworks that a
     * given Course has Access.
     * UNIMPLEMENTED
     * @param coursePk A BB course PK
     * @param startDate A datetime - the earliest time possible of a SlotSignup in this list. If null
     * then there's no early limit.
     * @param endDate A datetime - the latest time possible of a SlotSignup in this list. If null
     * then there's no late limit.
     * @return All SlotSignups for Blocks within the given time range on Frameworks that a
     * given Course has Access
     */
    public static SlotSignupList loadByCourseId (int coursePk, Date startDate, Date endDate)
    {
        SlotSignupList ssl = new SlotSignupList();

        return ssl;
    }

    /**
     * UNIMPLEMENTED
     * @param blockGroupId 
     * @return 
     */
    public static SlotSignupList loadByBlockGroupId (int blockGroupId)
    {
        return loadByBlockGroupId(blockGroupId,null,null);
    }

    /**
     * UNIMPLEMENTED
     * @param blockGroupId 
     * @param startDate 
     * @param endDate 
     * @return 
     */
    public static SlotSignupList loadByBlockGroupId (int blockGroupId, Date startDate, Date endDate)
    {
        SlotSignupList ssl = new SlotSignupList();

        return ssl;
    }

    /**
     * All the SlotSignups for a given Block
     * @param blockId A Block ID
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return All the SlotSignups for the given Block
     */
    public static SlotSignupList loadByBlockId (int blockId)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        String sqlText = 
            "SELECT " + 
            "ssslot.ss_slot_id, ssslot.date_created, ssslot.date_updated, " + 
            "ssslot.updated_by_user_pk1, ssslot.ss_block_id, ssslot.user_pk1, ssslot.admin_comment, " + 
            "u.user_id, u.firstname, u.lastname " + 
            "FROM wms_ss_vw_slot_nd ssslot, users u " + 
            "WHERE u.row_status = 0 " + 
            "AND u.pk1 = ssslot.user_pk1 " + 
            "AND ssslot.ss_block_id = " + Integer.toString(blockId) +
            " ORDER BY u.lastname, u.firstname";

        return createFromSql(sqlText);
    }
    
    /** private static methods */
    private static SlotSignupList createFromSql (String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
   {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        SlotSignupList ssl = new SlotSignupList();
        while (res.next())
        {
            SlotSignup s = new SlotSignup(res);
            ssl.add(s);
        }
        
        sql.close();        
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
        
        return ssl;
    }

    /** object methods */

    /** constructors */
    private SlotSignupList() {
    }
    
}
