/*
 * Connection.java
 *
 * Created on August 19, 2005, 11:23 AM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.db;

import blackboard.db.*;
import java.sql.*;
import java.text.SimpleDateFormat;

/**
 * This is a simlple class to ease acquisition of connection managers and 
 * actual connections, both using the Blackboard DB pool. Typical usage is 
 * something like this:
 * <CODE>
 *    ConnectionManager bbConnMgr = Connect.getConnectionManager();
 *    java.sql.Connection conn = Connect.getConnection(bbConnMgr);
 *    Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
 *    ResultSet res = sql.executeQuery("sql-statement-here");
 * 
 *    // do stuff with the results
 *    
 *    sql.close();            
 *    if(bbConnMgr != null) {
 *        bbConnMgr.releaseConnection(conn);
 *    }
 *    
 * </CODE>
 * @author cwarren
 */
public class Connect {

    /** static fields/tools */
    
    /**
     * This method is used to get a connection manager, which in turn is used to get 
     * connections.
     * @return A ConnectionManager object
     */

    public static ConnectionManager getConnectionManager()
    {
        BbDatabase bbDb = DbUtil.safeGetBbDatabase();
        ConnectionManager conman = bbDb.getConnectionManager();
        return conman;
    }
    
    /**
     * This is used to get a DB connection from the BB connection pool.
     * @param bbConnMgr A ConnectionManager object
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @return A Connection object - a connection to the database
     */
    public static java.sql.Connection getConnection(ConnectionManager bbConnMgr)
        throws ConnectionNotAvailableException, InterruptedException
    {
        java.sql.Connection conn = null;
        int i = 0;
        while(conn == null && i<10) {
            try {
                conn = bbConnMgr.getConnection();
            }
            catch(ConnectionNotAvailableException cnae){
                ++i;
                if (i >= 10) {
                    throw cnae;
                } else {
                    Thread.sleep(1000);
                }
            }
        }
        return conn;
    }
    
    
    /** object methods */

    /** constructors */
    private Connect() {
    }
    
}
