/*
 * SqlUtil.java
 *
 * Created on August 24, 2005, 12:05 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package wms.ss.db;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ArrayList;
import blackboard.db.*;

/**
 * A collection of utility functions and objects useful in dealing with SQL and 
 * DB connections.
 * @author cwarren
 */
public class SqlUtil {

    /**
     * A SimpleDateFormat object used to create human readable date-time strings 
     * of the form YYYY/MM/DD HH24:MI:SS (the format used when inserting and updating 
     * dates in the DB) from Date objects.
     */
    public static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy/MM/dd H:mm:ss");
    public static SimpleDateFormat checkConflictDateFormatter = new SimpleDateFormat("yyyy/MM/dd h:mm a");
    /**
     * The format mask/string used in the oracle SQL to_date function to convert strings
     * generated from Date objects using the dateFormatter into data suitable for use in
     * columns of type DATE.
     */
    public static String dateFormatStringForDB = "YYYY/MM/DD HH24:MI:SS";
    public static String dateFormatStringForDBConflictChecking = "YYYY/MM/DD HH:MI AM";
    
    /**
     * This generates the SQL necessary for checking a range of date-times. If nulls 
     * are passed in for the early or late part of the range (or both) then that part 
     * is not checked.
     * @param sqlText The base SQL as a string - the SQL to which the date-time range 
     * checking will be apended
     * @param startDateField The name of the field to be used in checking the earlier end of the time range
     * @param endDateField The name of the field to be used in checking the later end of the time range
     * @param startDate The early limit
     * @param endDate The late limit
     * @return A SQL statement, which is the original one that was passed in with 
     * any necessary conditions for checking the date-time range appended.
     */
    public static String appendDateRangeCheck(String sqlText, String startDateField, String endDateField, java.util.Date startDate, java.util.Date endDate)
    {
        if (startDate != null)
        {
            sqlText = sqlText + "AND "+startDateField+" >= to_date('"+ SqlUtil.dateFormatter.format(startDate)+"','YYYY/MM/DD HH24:MI:SS') ";
        }
        if (endDate != null)
        {
            sqlText = sqlText + "AND "+endDateField+" <= to_date('"+ SqlUtil.dateFormatter.format(endDate)+"','YYYY/MM/DD HH24:MI:SS') ";
        }
        return sqlText;
    }
    
    /*public static ResultSet quickQuery(java.sql.Connection conn, String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        return sql.executeQuery(sqlText);
    }*/
    
    /**
     * A utility function to get an int back from a SELECT satement that generates 
     * a single result row.
     * @param sqlText The SELECT statement to execute
     * @param fieldName The name of the field/column from which the return value is gotten
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The value of the indicated field of the first (only) result of the 
     * SELECT stement.
     */
    public static int runSqlGetIntField(String sqlText, String fieldName)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        int resInt = 0;
        if (res.next())
        {
            resInt = res.getInt(fieldName);
        }
        sql.close();
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return resInt;
    }
 
    /**
     * A utility function to get an String back from a SELECT satement that generates 
     * a single result row.
     * @param sqlText The SELECT statement to execute
     * @param fieldName The name of the field/column from which the return value is gotten
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     * @return The value of the indicated field of the first (only) result of the 
     * SELECT stement.
     */
    public static String runSqlGetStringField(String sqlText, String fieldName)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);

        String resStr = "";
        if (res.next())
        {
            resStr = res.getString(fieldName);
        }
        
        sql.close();
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }

        return resStr;
    }

    /**
     * A utility function to run an arbitrary SQL statement when you don't care about 
     * the result set. Typically used for INSERT, UPDATE, and DELETE.
     * @param sqlText The SQL statement to be executed
     * @throws blackboard.db.ConnectionNotAvailableException 
     * @throws java.lang.InterruptedException 
     * @throws java.sql.SQLException 
     */
    public static void runSql(String sqlText)
        throws ConnectionNotAvailableException, InterruptedException, SQLException
    {
        ConnectionManager bbConnMgr = Connect.getConnectionManager();
        java.sql.Connection conn = Connect.getConnection(bbConnMgr);
        Statement sql = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        ResultSet res = sql.executeQuery(sqlText);
        sql.close();
        if(bbConnMgr != null) {
            bbConnMgr.releaseConnection(conn);
        }
    }

    /** Creates a new instance of SqlUtil */
    private SqlUtil() {
    }
    
}
