/*
 * HtmlCalendarTool.java
 *
 * Created on August 26, 2005, 1:09 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 *
 * This object returns various HTML that's useful in displaying calendars in HTML
 */

package wms.ss.display;

import java.util.*;
import java.text.SimpleDateFormat;


/**
 * This class is used in the display of calendars in HTML. It also includes a 
 * few standard date formatting objects. It's used something like this:
 * <CODE>
 * BlockList blocks = BlockList.loadByFrameworkId(frameworkId);
 * HtmlCalendarTool hct = new HtmlCalendarTool();
 * Block lastBlock = null;
 * for (int blockIndex = 0; blockIndex < blocks.size(); blockIndex++)
 * {
 *    Block curBlock = (Block)blocks.get(blockIndex);
 *    String htmlForTimeBetween = "";
 *    if (lastBlock==null)
 *    {
 *        out.print(hct.startCalendar(curBlock.getStartDatetime()));
 *    }
 *    else
 *    {
 *        htmlForTimeBetween = hct.emptyTimeBetween(lastBlock.getEndDatetime(),curBlock.getStartDatetime());
 *        out.print(htmlForTimeBetween);
 *    }
 *    // display info for the block    
 *    lastBlock = curBlock;
 * }
 * if (blocks.size() > 0) {
 *    out.print(hct.endCalendar(lastBlock.getEndDatetime()));
 * }
 * </CODE>
 * @author cwarren
 */
public class HtmlCalendarTool {

    public static String pastDayTimingName = "pastDay";
    public static String presentOrFutureDayTimingName = "presentOrFutureDay";    
    /**
     * The font used in the calendar
     */
    public static String fontFamilyCss = " font-family:arial,sans-serif;";
    /**
     * the color of things in the past
     */
    public static String pastColor = "gray";
    /**
     * the color of things in the present or future
     */
    public static String pendingColor = "black";
    /**
     * color to denote the first of the month
     */
    public static String firstOfMonthColor = "blue";
    /**
     * kind of border for an empty calendar cell
     */
    public static String emptyBorder = "medium solid";
    /**
     * kind of border for a calendar cell that has stuff in it
     */
    public static String usedBorder = "thick solid";
    /**
     * font size to use for day of month (e.g. 5)
     */
    public static String dayOfMonthFontSize = "24pt";
    /**
     * font size to use for the name of the month (e.g. September)
     */
    public static String monthFontSize = "10pt";
    /**
     * font size to use for the day of the week (e.g. Friday)
     */
    public static String dayOfWeekFontSize = "8pt";
    /**
     * font size to use for cell contents
     */
    public static String dayTextFontSize = "8pt";

    
    /**
     * composite style
     */
    public static String emptyPastDayCellStyle = "border:"+emptyBorder+" "+pastColor+";";
    /**
     * composite style
     */
    public static String emptyDayCellStyle = "border:"+emptyBorder+" "+pendingColor+";";
    /**
     * composite style
     */
    public static String emptyPastDayDayOfMonthStyle = "float:left; font-weight:bold; font-size:"+dayOfMonthFontSize+";"+fontFamilyCss+" color:"+pastColor+";";
    /**
     * composite style
     */
    public static String emptyDayDayOfMonthStyle = "float:left; font-weight:bold; font-size:"+dayOfMonthFontSize+";"+fontFamilyCss+" color:"+pendingColor+";";
    /**
     * composite style
     */
    public static String emptyPastDayDayOfWeekStyle = "font-size: "+dayOfWeekFontSize+"; color: "+pastColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String emptyDayDayOfWeekStyle = "font-size: "+dayOfWeekFontSize+"; color: "+pendingColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String emptyPastDayMonthStyle = "font-size: "+monthFontSize+"; color: "+pastColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String emptyDayMonthStyle = "font-size: "+monthFontSize+"; color: "+pendingColor+";"+fontFamilyCss;

    /**
     * composite style
     */
    public static String usedPastDayCellStyle = "border:"+usedBorder+" "+pastColor+";";
    /**
     * composite style
     */
    public static String usedDayCellStyle = "border:"+usedBorder+" "+pendingColor+";";
    /**
     * composite style
     */
    public static String usedPastDayDayOfMonthStyle = "float:left; font-weight:bold; font-size:"+dayOfMonthFontSize+";"+fontFamilyCss+" color:"+pastColor+";";
    /**
     * composite style
     */
    public static String usedDayDayOfMonthStyle = "float:left; font-weight:bold; font-size:"+dayOfMonthFontSize+";"+fontFamilyCss+" color:"+pendingColor+";";
    /**
     * composite style
     */
    public static String usedPastDayDayOfWeekStyle = "font-size: "+dayOfWeekFontSize+"; color: "+pastColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String usedDayDayOfWeekStyle = "font-size: "+dayOfWeekFontSize+"; color: "+pendingColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String usedPastDayMonthStyle = "font-size: "+monthFontSize+"; color: "+pastColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String usedDayMonthStyle = "font-size: "+monthFontSize+"; color: "+pendingColor+";"+fontFamilyCss;

    /**
     * composite style
     */
    public static String pastDayTextStyle = "font-size:"+dayTextFontSize+"; color:"+pastColor+";" + fontFamilyCss;
    /**
     * composite style
     */
    public static String dayTextStyle = "font-size:"+dayTextFontSize+"; color:"+pendingColor+";" + fontFamilyCss;

    /**
     * composite style
     */
    public static String dayFirstDayOfMonthStyle = "float:left; font-weight:bold; font-size:"+dayOfMonthFontSize+";"+fontFamilyCss+" color:"+firstOfMonthColor+";";
    /**
     * composite style
     */
    public static String monthFirstDayOfMonthStyle = "font-size: "+monthFontSize+"; color: "+firstOfMonthColor+";"+fontFamilyCss;
    /**
     * composite style
     */
    public static String cellTodayStyle = "background-color:#BBFFBB;";    

    /**
     * width of a day cell in pixels
     */
    public static String dayCellWidth = "100";
    /**
     * html code to close a cell
     */
    public static String endDayCellCode = "  </p>\n  </div>\n  </td>\n";
    /**
     * html code to start a week
     */
    public static String beginWeekCode = "\n <tr>";
    /**
     * html code to end a week
     */
    public static String endWeekCode = " </tr>\n";
    
    /**
     * date formatter for YYYY/MM/DD form
     */
    public static SimpleDateFormat downToDayDateFormatter = new SimpleDateFormat("yyyy/MM/dd");
    /**
     * date formatter for getting the day of the month
     */
    public static SimpleDateFormat dayOfMonthDateFormatter = new SimpleDateFormat("d");
    /**
     * date formatter for getting the day of the week
     */
    public static SimpleDateFormat dayOfWeekDateFormatter = new SimpleDateFormat("EEEE");
    /**
     * date formatter for getting the month (full month name)
     */
    public static SimpleDateFormat monthFormatter = new SimpleDateFormat("MMMM");
    /**
     * date formatter for getting the hour and minute (X:XX)
     */
    public static SimpleDateFormat hourMinFormatter = new SimpleDateFormat("h:mm");
    /**
     * date formatter for getting the hour and minute and AM/PM indicator (X:XX XM)
     */
    public static SimpleDateFormat hourMinAmPmFormatter = new SimpleDateFormat("h:mm a");

    private int firstDayOfWeek = Calendar.SUNDAY;
    private int lastDayOfWeek = Calendar.SATURDAY;

    private String curCellStyle = "";
    private String curDayOfMonthStyle = "";
    private String curMonthStyle = "";
    private String curDayDayOfWeekStyle = "";
    private String curDayTextStyle = "";
    
    /**
     * Creates a new instance of HtmlCalendarTool
     */
    public HtmlCalendarTool() {
     }

    /**
     * Returns the HTML necessary to start a calendar with the first info on the given date.
     * It starts on the week that contains the given date, with cells up to that date having
     * no day text (i.e. the cells have the day number, month, and day of week, but nothing
     * indicated as happening on that day).
     * @param firstDay The Date object specifying the first date and time on which something is happening
     * @return The HTML necessary to start a calendar (uses standard html table)
     */
    public String startCalendar(java.util.Date firstDay)
    {
        return "<table>" + beginWeekCode + emptyDaysFromBeginningOfWeek(firstDay) + startUsedDayFor(firstDay);
    }
    
    /**
     * Returns the HTML necessary to finish a calendar which had the last event on the
     * indicated day.
     * @param lastDay The last day on which an event occurred
     * @return The HTML necessary to end the table.
     */
    public String endCalendar(java.util.Date lastDay)
    {
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(lastDay);
        if (fromCal.get(Calendar.DAY_OF_WEEK) == lastDayOfWeek)
        {
            return endDayCellCode + endWeekCode + "</table>";
        }
        //fromCal.add(Calendar.DATE, 1);
        return endDayCellCode + emptyDaysToEndOfWeek(fromCal.getTime()) + endWeekCode + "</table>";
    }

    /**
     * Generates HTML for empty (i.e. has no events) days from one day to the next. The 
     * fromDay is close, the intervening days (and weeks if necessary) are created, and 
     * the tilDay is started. If from and til are the same day, returns nothing.
     * @param fromDay The last date-time something happened
     * @param tilDay the next datetime something happens
     * @return the HTML to close the previous day, cover all the time between (from nothing for same day, to days or weeks or months), and start the new day
     */
    public String emptyTimeBetween(java.util.Date fromDay, java.util.Date tilDay)
    {
        // same day - no time between to return
        if (downToDayDateFormatter.format(fromDay).equals(downToDayDateFormatter.format(tilDay)))
        {
            return "";
        }
        
        // later that week - return end of fromDay and intervening empty days and start of used tilDay
        // check til day of week is > from day of week AND time between from day and til day is < 7
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(fromDay);
        Calendar tilCal = Calendar.getInstance();
        tilCal.setTime(tilDay);
        Calendar comparingTilCal = Calendar.getInstance();
        comparingTilCal.setTime(tilDay);
        comparingTilCal.add(Calendar.DAY_OF_MONTH, -7);
        if ((tilCal.get(Calendar.DAY_OF_WEEK) > fromCal.get(Calendar.DAY_OF_WEEK))
            && (fromCal.after(comparingTilCal)))
        {
            fromCal.add(Calendar.DATE, 1);
            return endDayCellCode + emptyDaysBetween(fromCal.getTime(), tilDay) + startUsedDayFor(tilDay);
        }
        
        // spans different weeks
        //fromCal.add(Calendar.DATE, 1);
        String theHtml = endDayCellCode + emptyDaysToEndOfWeek(fromCal.getTime()) + endWeekCode;
        while (fromCal.get(Calendar.DAY_OF_WEEK) > firstDayOfWeek)
        {
            fromCal.add(Calendar.DATE, -1);
        }
        fromCal.add(Calendar.DATE, 7);
        comparingTilCal.setTime(tilDay);
        while (comparingTilCal.get(Calendar.DAY_OF_WEEK) > firstDayOfWeek)
        {
            comparingTilCal.add(Calendar.DATE, -1);
        }
        // at this point the fromCal is at the start of the week after fromDay, and comparingTilCal is at the beginning of the week containing tilDay
        // as long as fromCal(date) is less than comparingTilCal(date), we can add a week
        while (downToDayDateFormatter.format(fromCal.getTime()).compareTo(downToDayDateFormatter.format(comparingTilCal.getTime())) < 0)
        //while (fromCal.before(comparingTilCal))
        {
//            theHtml = theHtml + "(from: "+downToDayDateFormatter.format(fromCal.getTime())
//                +", til"+downToDayDateFormatter.format(comparingTilCal.getTime())+")" 
            theHtml = theHtml + emptyWeekStartingOn(fromCal.getTime());
            fromCal.add(Calendar.DATE, 7);
            }
        theHtml = theHtml + beginWeekCode + emptyDaysFromBeginningOfWeek(tilDay) + startUsedDayFor(tilDay);
        return theHtml;
    }

    /**
     * Generates HTML for a week of empty days stating on the given date.
     * @param fromDay The date that starts the week (a Sunday)
     * @return the HMTL for the empty week
     */
    public String emptyWeekStartingOn(java.util.Date fromDay)
    {
        String theHtml = beginWeekCode;
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(fromDay);
        for (int i = 0; i < 7; i++)
        {
            theHtml = theHtml + emptyDayFor(fromCal.getTime());
            fromCal.add(Calendar.DATE, 1);
        }
        theHtml = theHtml + endWeekCode;
        
        return theHtml;
    }
    
    /* used if the two dates are in the same week */
    // includes fromDay and NOT tilDay
    /**
     * Generates HTML for empty days between two dates in the same week. Includes fromDay 
     * and NOT tilDay.
     * @param fromDay Starting on this day (inclusive)
     * @param tilDay ending on this day (exclusive)
     * @return HTML for the intervening empty days
     */
    public String emptyDaysBetween(java.util.Date fromDay, java.util.Date tilDay)
    {
        String theHtml = "";
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(fromDay);
        Calendar tilCal = Calendar.getInstance();
        tilCal.setTime(tilDay);
        //tilCal.add(Calendar.DATE, -1);
        while (downToDayDateFormatter.format(fromCal.getTime()).compareTo(downToDayDateFormatter.format(tilCal.getTime())) < 0)
        {
            theHtml = theHtml + emptyDayFor(fromCal.getTime());
            fromCal.add(Calendar.DATE, 1);
        }
        return theHtml;
    }

    /**
     * Generates html for empty days from the given day to the end of that week.
     * @param fromDay starting on this day (inclusive)
     * @return HTML for empty days til the end of the week
     */
    public String emptyDaysToEndOfWeek(java.util.Date fromDay)
    {
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(fromDay);
        Calendar tilCal = Calendar.getInstance();
        tilCal.setTime(fromDay);
        if (tilCal.get(Calendar.DAY_OF_WEEK) == lastDayOfWeek)
        {
            return "";
        }
        while (tilCal.get(Calendar.DAY_OF_WEEK) < lastDayOfWeek)
        {
            tilCal.add(Calendar.DATE, 1);
        }
        tilCal.add(Calendar.DATE, 1); // needed since the til day is not included in emptyDaysBetween
        fromCal.add(Calendar.DATE, 1); // needed since the fromDay IS included in emptyDaysBetween
        return emptyDaysBetween(fromCal.getTime(), tilCal.getTime());
    }

    // includes Sunday, does not include tilDay
    /**
     * Generates html for the week that includes the given date, from Sunday to that day (exlcusive)
     * @param tilDay from sunday til this day (exclusive)
     * @return HTML for the beginning of the week
     */
    public String emptyDaysFromBeginningOfWeek(java.util.Date tilDay)
    {
        Calendar fromCal = Calendar.getInstance();
        fromCal.setTime(tilDay);
        while (fromCal.get(Calendar.DAY_OF_WEEK) > firstDayOfWeek)
        {
            fromCal.add(Calendar.DATE, -1);
        }
        return emptyDaysBetween(fromCal.getTime(), tilDay);
    }

    
    /**
     * Generates HTML for an empty day
     * @param forDay The date for which the day cell is generated
     * @return The HTML for an empty day cell for the specified date
     */
    public String emptyDayFor(java.util.Date forDay)
    {
        return startEmptyDayFor(forDay)+endDayCellCode;
    }
    
    /**
     * Generates HTML for the start of an empty day
     * @param forDay for cell generated for this day
     * @return HTML for the start of an empty/unused day
     */
    public String startEmptyDayFor(java.util.Date forDay)
    {
        curCellStyle = emptyDayCellStyle;
        curDayOfMonthStyle = emptyDayDayOfMonthStyle;
        curMonthStyle = emptyDayMonthStyle;
        curDayDayOfWeekStyle = emptyDayDayOfWeekStyle;
        curDayTextStyle = dayTextStyle;

        String forDayStr = downToDayDateFormatter.format(forDay);
        int compareRes = forDayStr.compareTo(downToDayDateFormatter.format(Calendar.getInstance().getTime()));
        if (compareRes < 0)
        {
            curCellStyle = emptyPastDayCellStyle;
            curDayOfMonthStyle = emptyPastDayDayOfMonthStyle;
            curMonthStyle = emptyPastDayMonthStyle;
            curDayDayOfWeekStyle = emptyPastDayDayOfWeekStyle;
            curDayTextStyle = pastDayTextStyle;
        } 
        else if (compareRes == 0) 
        {
            curCellStyle = emptyDayCellStyle + " " + cellTodayStyle;
        }
        
        if (dayOfMonthDateFormatter.format(forDay).equalsIgnoreCase("1"))
        {
            curDayOfMonthStyle = dayFirstDayOfMonthStyle;
            curMonthStyle = monthFirstDayOfMonthStyle;
        }
        
        return createDayStart(
                dayOfMonthDateFormatter.format(forDay), 
                monthFormatter.format(forDay), 
                dayOfWeekDateFormatter.format(forDay));
    }

    /**
     * Generates HTML for the start of a day on which something is happening
     * @param forDay A day on which something is happening
     * @return The HTML that starts a day cell for a day on which something happens
     */
    public String startUsedDayFor(java.util.Date forDay)
    {
        curCellStyle = usedDayCellStyle;
        curDayOfMonthStyle = usedDayDayOfMonthStyle;
        curMonthStyle = usedDayMonthStyle;
        curDayDayOfWeekStyle = usedDayDayOfWeekStyle;
        curDayTextStyle = dayTextStyle;
       
        String forDayStr = downToDayDateFormatter.format(forDay);
        int compareRes = forDayStr.compareTo(downToDayDateFormatter.format(Calendar.getInstance().getTime()));
        if (compareRes < 0)
        {
            curCellStyle = usedPastDayCellStyle;
            curDayOfMonthStyle = usedPastDayDayOfMonthStyle;
            curMonthStyle = usedPastDayMonthStyle;
            curDayDayOfWeekStyle = usedPastDayDayOfWeekStyle;
            curDayTextStyle = pastDayTextStyle;
        } 
        else if (compareRes == 0) 
        {
            curCellStyle = usedDayCellStyle + " " + cellTodayStyle;
        }
        
         if (dayOfMonthDateFormatter.format(forDay).equalsIgnoreCase("1"))
        {
            curDayOfMonthStyle = dayFirstDayOfMonthStyle;
            curMonthStyle = monthFirstDayOfMonthStyle;
        }
        
        return createDayStart(
                dayOfMonthDateFormatter.format(forDay), 
                monthFormatter.format(forDay), 
                dayOfWeekDateFormatter.format(forDay));
    }

    private String createDayStart(String dayOfMonth, String month, String dayOfWeek)
    {
        String dayTimingName = presentOrFutureDayTimingName;
        if (curDayTextStyle.equals(pastDayTextStyle))
        {
            dayTimingName = pastDayTimingName;
        }
        
        return " <td valign=\"top\" align=\"left\" width=\""+HtmlCalendarTool.dayCellWidth+"\" style=\""+curCellStyle+"\">\n"+
                " <div name=\""+dayTimingName+"\">\n"+
                "  <span style=\""+curDayOfMonthStyle+"\">"+dayOfMonth+"</span>\n"+
                "  <span style=\""+curMonthStyle+"\">"+month+"</span><br>\n"+
                "  <span style=\""+curDayDayOfWeekStyle+"\">"+dayOfWeek+"</span><br>\n"+
                "  <p style=\""+curDayTextStyle+"\">\n";
    }
    
    /**
     * Gets the style currently used to generate days. Used if, e.g., one wants to puts text in a day cell and want the text style to be appropriate for that day (e.g. if the day is in the past).
     * @return a style as per CSS (appropriate for use in a style attribute)
     */
    public String getCurCellStyle() {
        return curCellStyle;
    }

    /**
     * Sets the style of the current cell
     * @param curCellStyle a style as per CSS (appropriate for use in a style attribute)
     */
    public void setCurCellStyle(String curCellStyle) {
        this.curCellStyle = curCellStyle;
    }

    /**
     * Gets the style currently being used for the day of the month
     * @return a style as per CSS (appropriate for use in a style attribute)
     */
    public String getCurDayOfMonthStyle() {
        return curDayOfMonthStyle;
    }

    /**
     * Sets the style currently being used for the day of the month
     * @param curDayOfMonthStyle a style as per CSS (appropriate for use in a style attribute)
     */
    public void setCurDayOfMonthStyle(String curDayOfMonthStyle) {
        this.curDayOfMonthStyle = curDayOfMonthStyle;
    }

    /**
     * Gets the style used for the name for the name of the month
     * @return a style as per CSS (appropriate for use in a style attribute)
     */
    public String getCurMonthStyle() {
        return curMonthStyle;
    }

    /**
     * Sets the style used for the name for the name of the month
     * @param curMonthStyle a style as per CSS (appropriate for use in a style attribute)
     */
    public void setCurMonthStyle(String curMonthStyle) {
        this.curMonthStyle = curMonthStyle;
    }

    /**
     * Gets the style used for the day of the week
     * @return a style as per CSS (appropriate for use in a style attribute)
     */
    public String getCurDayDayOfWeekStyle() {
        return curDayDayOfWeekStyle;
    }

    /**
     * Sets the style used for the day of the week
     * @param curDayDayOfWeekStyle a style as per CSS (appropriate for use in a style attribute)
     */
    public void setCurDayDayOfWeekStyle(String curDayDayOfWeekStyle) {
        this.curDayDayOfWeekStyle = curDayDayOfWeekStyle;
    }

    /**
     * Gets the style used for the day cell contents
     * @return a style as per CSS (appropriate for use in a style attribute)
     */
    public String getCurDayTextStyle() {
        return curDayTextStyle;
    }

    /**
     * Sets the style used for the day cell contents
     * @param curDayTextStyle a style as per CSS (appropriate for use in a style attribute)
     */
    public void setCurDayTextStyle(String curDayTextStyle) {
        this.curDayTextStyle = curDayTextStyle;
    }
}
